@@ -1,15 +1,15 @@
  /*
  * Project       : ipv6calc
  * File          : dbipv6_assignment.h
- * Version       : $Id: dbipv6addr_assignment.h,v 1.37 2012/02/05 09:33:55 peter Exp $
- * Generated     : Sun Feb  5 11:08:35 2012
+ * Version       : $Id: dbipv6addr_assignment.h,v 1.45 2013/04/10 20:28:51 ds6peter Exp $
+ * Generated     : Wed Apr 10 20:42:28 2013
  * Data copyright: IANA ARIN RIPENCC APNIC LACNIC AFRINIC
  *
  * Information:
  *  Additional header file for libipv6addr.c
  */
 
-static const char* dbipv6addr_registry_status = "AFRINIC/20120205 APNIC/20120203 ARIN/20120204 IANA/20080827 LACNIC/20120203 RIPENCC/20120203";
+/*@unused@*/ static const char* dbipv6addr_registry_status __attribute__ ((__unused__)) = "AFRINIC/20130410 APNIC/20130410 ARIN/20130409 IANA/20130215 LACNIC/20130409 RIPENCC/20130409";
 
 static const s_ipv6addr_assignment dbipv6addr_assignment[] = {
 	{ 0x20010000, 0x00000000, 0xfffffe00, 0x00000000,  23, "IANA" },
@@ -73,4 +73,6 @@ static const s_ipv6addr_assignment dbipv6addr_assignment[] = {
 	{ 0x2d000000, 0x00000000, 0xff000000, 0x00000000,   8, "IANA" },
 	{ 0x2e000000, 0x00000000, 0xfe000000, 0x00000000,   7, "IANA" },
 	{ 0x30000000, 0x00000000, 0xf0000000, 0x00000000,   4, "IANA" },
+	{ 0x3ffe0000, 0x00000000, 0xffff0000, 0x00000000,  16, "IANA" },
+	{ 0x5f000000, 0x00000000, 0xff000000, 0x00000000,   8, "IANA" },
 };
