@@ -1,8 +1,8 @@
 /*
  * Project    : ipv6calc
  * File       : ipv6calc.c
- * Version    : $Id: ipv6calc.c,v 1.54 2011/11/26 16:07:23 peter Exp $
- * Copyright  : 2001-2011 by Peter Bieringer <pb (at) bieringer.de>
+ * Version    : $Id: ipv6calc.c,v 1.78 2013/05/12 07:34:04 ds6peter Exp $
+ * Copyright  : 2001-2013 by Peter Bieringer <pb (at) bieringer.de>
  * 
  * Information:
  *  Central program (main)
@@ -24,6 +24,7 @@
 #include "libipv4addr.h"
 #include "libipv6addr.h"
 #include "libmac.h"
+#include "libeui64.h"
 
 #include "showinfo.h"
 
@@ -39,6 +40,7 @@
 #include "config.h"
 #include "libieee.h"
 #include "../databases/ieee-oui/dbieee_oui.h"
+#include "../databases/ieee-oui36/dbieee_oui36.h"
 #include "../databases/ieee-iab/dbieee_iab.h"
 #include "libipv4addr.h"
 #include "../databases/ipv4-assignment/dbipv4addr_assignment.h"
@@ -69,9 +71,11 @@ int input_is_pipe = 0;
 #define LINEBUFFER      16384
 
 /* anonymization default values */
-int mask_ipv4 = 24;
-int mask_ipv6 = 48;
-int mask_iid = 1;
+s_ipv6calc_anon_set ipv6calc_anon_set;
+int mask_ipv4;
+int mask_ipv6;
+int mask_iid;
+int mask_mac;
 
 int  use_ip2location_ipv4 = 0; /* if set to 1, IP2Location IPv4 is enabled by option(s) */
 int  use_ip2location_ipv6 = 0; /* if set to 1, IP2Location IPv6 is enabled by option(s) */
@@ -131,7 +135,7 @@ void printversion_verbose(void) {
 	fprintf(stderr, "\n");
 
 #ifdef SUPPORT_DB_IEEE
-	fprintf(stderr, "IEEE database included: %s %s\n", libieee_iab_status, libieee_oui_status);
+	fprintf(stderr, "IEEE database included: %s %s %s\n", libieee_iab_status, libieee_oui_status, libieee_oui36_status);
 #else
 	fprintf(stderr, "IEEE database not included\n");
 #endif
@@ -155,7 +159,7 @@ void printversion_verbose(void) {
 #if defined (SUPPORT_GEOIP_COUNTRY_CODE_BY_ADDR_V6) && defined (SUPPORT_GEOIP_COUNTRY_NAME_BY_ADDR_V6)
 	fprintf(stderr, "GeoIP support enabled, compiled with IPv4 & IPv6 support\n");
 #else
-	fprintf(stderr, "GeoIP support enabled, compiled with IPv4 & IPv6 support (in compatibilty mode)\n");
+	fprintf(stderr, "GeoIP support enabled, compiled with IPv4 & IPv6 support (in compatibility mode)\n");
 #endif
 #else
 	fprintf(stderr, "GeoIP support enabled, compiled with IPv4 support only\n");
@@ -220,7 +224,7 @@ int main(int argc, char *argv[]) {
 	char resultstring[NI_MAXHOST] = "";
 	char resultstring2[NI_MAXHOST] = "";
 	char resultstring3[NI_MAXHOST] = "";
-	int retval = 1, i, j, lop;
+	int retval = 1, i, j, lop, result;
 	unsigned long int command = 0;
 	int bit_start = 0, bit_end = 0, force_prefix = 0;
 	char *input1 = NULL, *input2 = NULL;
@@ -239,6 +243,15 @@ int main(int argc, char *argv[]) {
 	ipv6calc_ipv6addr ipv6addr, ipv6addr2, ipv6addr3, ipv6addr4;
 	ipv6calc_ipv4addr ipv4addr, ipv4addr2;
 	ipv6calc_macaddr  macaddr;
+	ipv6calc_eui64addr  eui64addr;
+
+	/* filter structure */
+	s_ipv6calc_filter_master filter_master;
+	libipv6calc_filter_clear(&filter_master);
+
+	/* IID statistics */
+	s_iid_statistics iid_statistics;
+	int iid_privacy_result;
 
 	/* clear address structures */
 	ipv6addr_clearall(&ipv6addr);
@@ -256,6 +269,13 @@ int main(int argc, char *argv[]) {
 	int linecounter = 0;
 	int flush_mode = 0;
 
+	/* default */
+	result = libipv6calc_anon_set_by_name(&ipv6calc_anon_set, "anonymize-standard");
+	if (result != 0) {
+		fprintf(stderr, "major problem, ipv6calc anonymization default preset not found: anonymize-standard\n");
+		exit(EXIT_FAILURE);
+	};
+
 	if (argc <= 1) {
 		printinfo();
 		exit(EXIT_FAILURE);
@@ -283,10 +303,23 @@ int main(int argc, char *argv[]) {
 			case 'h':
 			case '?':
 				command |= CMD_printhelp;
+				fprintf(stderr, "%s: help option detected\n", DEBUG_function_name);
 				break;
 				
 			case 'd':
-				ipv6calc_debug = atol(optarg);
+				if ((strlen(optarg) > 2) && ((strncmp(optarg, "0x", 2) == 0) || (strncmp(optarg, "0X", 2)) == 0)) {
+					// convert hex
+					if (sscanf(optarg + 2, "%lx", &ipv6calc_debug) == 0) {
+						ipv6calc_debug = 0;
+						fprintf(stderr, "%s: can't parse value for debug option: %s\n", DEBUG_function_name, optarg);
+						exit (1);
+					} else {
+					};
+				} else {
+					ipv6calc_debug = atol(optarg);
+				};
+
+				fprintf(stderr, "%s: given debug value: %lx\n", DEBUG_function_name, ipv6calc_debug);
 				break;
 
 			case DB_ip2location_ipv4:
@@ -294,7 +327,7 @@ int main(int argc, char *argv[]) {
 				if (ipv6calc_debug != 0) {
 					fprintf(stderr, "%s: Got IP2Location IPv4 database file: %s\n", DEBUG_function_name, optarg);
 				};
-				strncpy(file_ip2location_ipv4, optarg, sizeof(file_ip2location_ipv4) -1 );
+				snprintf(file_ip2location_ipv4, sizeof(file_ip2location_ipv4) - 1, "%s", optarg);
 				use_ip2location_ipv4 = 1;
 #else
 				if ((formatoptions & FORMATOPTION_quiet) == 0) {
@@ -308,7 +341,7 @@ int main(int argc, char *argv[]) {
 				if (ipv6calc_debug != 0) {
 					fprintf(stderr, "%s: Got IP2Location IPv6 database file: %s\n", DEBUG_function_name, optarg);
 				};
-				strncpy(file_ip2location_ipv6, optarg, sizeof(file_ip2location_ipv6) -1 );
+				snprintf(file_ip2location_ipv6, sizeof(file_ip2location_ipv6) - 1, "%s", optarg);
 				use_ip2location_ipv6 = 1;
 #else
 				if ((formatoptions & FORMATOPTION_quiet) == 0) {
@@ -356,7 +389,7 @@ int main(int argc, char *argv[]) {
 				if (ipv6calc_debug != 0) {
 					fprintf(stderr, "%s: Got GeoIP IPv4 database file: %s\n", DEBUG_function_name, optarg);
 				};
-				strncpy(file_geoip_ipv4, optarg, sizeof(file_geoip_ipv4) -1 );
+				snprintf(file_geoip_ipv4, sizeof(file_geoip_ipv4) - 1, "%s", optarg);
 				use_geoip_ipv4 = 1;
 #else
 				if ((formatoptions & FORMATOPTION_quiet) == 0) {
@@ -370,7 +403,7 @@ int main(int argc, char *argv[]) {
 				if (ipv6calc_debug != 0) {
 					fprintf(stderr, "%s: Got GeoIP IPv6 database file: %s\n", DEBUG_function_name, optarg);
 				};
-				strncpy(file_geoip_ipv6, optarg, sizeof(file_geoip_ipv6) -1 );
+				snprintf(file_geoip_ipv6, sizeof(file_geoip_ipv6) - 1, "%s", optarg);
 				use_geoip_ipv6 = 1;
 #else
 				if ((formatoptions & FORMATOPTION_quiet) == 0) {
@@ -523,8 +556,15 @@ int main(int argc, char *argv[]) {
 				command = CMD_showinfotypes;
 				break;
 
-			case CMD_ANON_NO_MASK_IID:
-				mask_iid = 0;
+			case CMD_ANON_MASK_IID:
+				mask_iid = 1;
+				mask_iid = atoi(optarg);
+				if (mask_iid < 0 || mask_iid > 64) {
+					fprintf(stderr, " value for option 'mask-iid' out-of-range  [0-64]\n");
+					exit(EXIT_FAILURE);
+				};
+				ipv6calc_anon_set.mask_iid = mask_iid;
+				snprintf(ipv6calc_anon_set.name, sizeof(ipv6calc_anon_set.name) - 1, "%s", "custom");
 				break;
 
 			case CMD_ANON_MASK_IPV4:
@@ -533,32 +573,75 @@ int main(int argc, char *argv[]) {
 					fprintf(stderr, " value for option 'mask-ipv4' out-of-range  [0-32]\n");
 					exit(EXIT_FAILURE);
 				};
+				ipv6calc_anon_set.mask_ipv4 = mask_ipv4;
+				snprintf(ipv6calc_anon_set.name, sizeof(ipv6calc_anon_set.name) - 1, "%s", "custom");
 				break;
 
 			case CMD_ANON_MASK_IPV6:
 				mask_ipv6 = atoi(optarg);
-				if (mask_ipv6 < 0 || mask_ipv6 > 32) {
-					fprintf(stderr, " value for option 'mask-ipv4' out-of-range  [0-64]\n");
+				if (mask_ipv6 < 0 || mask_ipv6 > 64) {
+					fprintf(stderr, " value for option 'mask-ipv6' out-of-range  [0-64]\n");
+					exit(EXIT_FAILURE);
+				};
+				ipv6calc_anon_set.mask_ipv6 = mask_ipv6;
+				snprintf(ipv6calc_anon_set.name, sizeof(ipv6calc_anon_set.name) - 1, "%s", "custom");
+				break;
+
+			case CMD_ANON_MASK_MAC:
+				mask_mac = atoi(optarg);
+				if (mask_mac < 0 || mask_mac > 48) {
+					fprintf(stderr, " value for option 'mask-mac' out-of-range  [0-48]\n");
+					exit(EXIT_FAILURE);
+				};
+				ipv6calc_anon_set.mask_mac = mask_mac;
+				snprintf(ipv6calc_anon_set.name, sizeof(ipv6calc_anon_set.name) - 1, "%s", "custom");
+				break;
+
+			case CMD_ANON_PRESET_STANDARD:
+				result = libipv6calc_anon_set_by_name(&ipv6calc_anon_set, "as");
+				if (result != 0) {
+					fprintf(stderr, "ipv6calc anonymization preset not found: anonymize-standard\n");
 					exit(EXIT_FAILURE);
 				};
 				break;
 
-			case CMD_ANON_PRESET_STANDARD:
-				mask_ipv4 = 24;
-				mask_ipv6 = 48;
-				mask_iid = 1;
-				break;
-
 			case CMD_ANON_PRESET_CAREFUL:
-				mask_ipv4 = 20;
-				mask_ipv6 = 40;
-				mask_iid = 1;
+				result = libipv6calc_anon_set_by_name(&ipv6calc_anon_set, "ac");
+				if (result != 0) {
+					fprintf(stderr, "ipv6calc anonymization preset not found: anonymize-careful\n");
+					exit(EXIT_FAILURE);
+				};
 				break;
 
 			case CMD_ANON_PRESET_PARANOID:
-				mask_ipv4 = 16;
-				mask_ipv6 = 32;
-				mask_iid = 1;
+				result = libipv6calc_anon_set_by_name(&ipv6calc_anon_set, "ap");
+				if (result != 0) {
+					fprintf(stderr, "ipv6calc anonymization preset not found: anonymize-paranoid\n");
+					exit(EXIT_FAILURE);
+				};
+				break;
+
+			case CMD_ANON_PRESET_OPTION:
+				result = libipv6calc_anon_set_by_name(&ipv6calc_anon_set, optarg);
+				if (result != 0) {
+					fprintf(stderr, "ipv6calc anonymization preset not found: %s\n", optarg);
+					exit(EXIT_FAILURE);
+				};
+				break;
+
+			case CMD_ANON_METHOD_OPTION:
+				for (i = 0; i < sizeof(ipv6calc_anon_methods) / sizeof(s_ipv6calc_anon_methods); i++) {
+					if (strcmp(ipv6calc_anon_methods[i].name, optarg) == 0) {
+						ipv6calc_anon_set.method = ipv6calc_anon_methods[i].method;
+						snprintf(ipv6calc_anon_set.name, sizeof(ipv6calc_anon_set.name) -1, "%s", "custom");
+						break;
+					};
+				};
+
+				if (i == sizeof(ipv6calc_anon_methods) / sizeof(s_ipv6calc_anon_methods)) {
+					fprintf(stderr, "ipv6calc anonymization method not supported: %s\n", optarg);
+					exit(EXIT_FAILURE);
+				};
 				break;
 
 			case CMD_6rd_relay_prefix:
@@ -658,6 +741,11 @@ int main(int argc, char *argv[]) {
 				formatoptions |= FORMATOPTION_printmirrored;
 				break;
 
+			case FORMATOPTION_NUM_print_iid_var + FORMATOPTION_NUM_HEAD:
+				formatoptions |= FORMATOPTION_print_iid_var;
+	       			formatoptions |= FORMATOPTION_printfulluncompressed;
+				break;
+
 			/* new options */
 			case 'I':	
 			case CMD_inputtype:
@@ -715,6 +803,24 @@ int main(int argc, char *argv[]) {
 				};
 				action_given = 1;
 				break;
+
+			case 'E':	
+				if (ipv6calc_debug != 0) {
+					fprintf(stderr, "%s: Got expression string: %s\n", DEBUG_function_name, optarg);
+				};
+
+				if ((strcmp(optarg, "-?") == 0) || (strcmp(optarg, "-h") == 0) || (strcmp(optarg, "--help") == 0) ) {
+					action = ACTION_filter;
+					command = CMD_printhelp;
+					break;
+				};
+
+				result = libipv6calc_filter_parse(optarg, &filter_master);
+
+				if (result != 0) {
+					exit(EXIT_FAILURE);
+				};
+				break;
 				
 			default:
 				fprintf(stderr, "Usage: (see '%s --command -?|-h|--help' for more help)\n", PROGRAM_NAME);
@@ -742,6 +848,9 @@ int main(int argc, char *argv[]) {
 		} else if (action == ACTION_auto) {
 			printhelp_actiontypes(formatoptions);
 			exit(EXIT_FAILURE);
+		} else if (action != ACTION_undefined) {
+			printhelp_action_dispatcher(action, 0);
+			exit(EXIT_FAILURE);
 		};
 
 	} else if (command == CMD_printexamples) {
@@ -840,6 +949,11 @@ PIPE_input:
 			fprintf(stderr, "Line empty: %d\n", linecounter);
 			goto PIPE_input;
 		};
+
+		/* remove trailing \n */
+		if (linebuffer[strlen(linebuffer) - 1] == '\n') {
+			linebuffer[strlen(linebuffer) - 1] = '\0';
+		};
 		
 		if (ipv6calc_debug != 0) {
 			fprintf(stderr, "%s: stdin got line: '%s'\n", DEBUG_function_name, linebuffer);
@@ -892,6 +1006,11 @@ PIPE_input:
 		};
 	};
 	
+	/* reset input type in case of action=filter and pipe mode */
+	if (input_is_pipe == 1 && action_given == 1 && action == ACTION_filter) {
+		inputtype = 0;
+	};
+
 	/* autodetection */
 	if ((inputtype == FORMAT_undefined || inputtype == FORMAT_auto) && inputc > 0) {
 		if (ipv6calc_debug != 0) {
@@ -965,11 +1084,15 @@ PIPE_input:
 			continue;
 		};
 
-		if ( (ipv6calc_outputformatoptionmap[i][1] & (formatoptions & ~ FORMATOPTION_quiet) ) == (formatoptions & ~ FORMATOPTION_quiet ) ) {
+		if ( (ipv6calc_outputformatoptionmap[i][1] & (formatoptions & ~FORMATOPTION_quiet) ) == (formatoptions & ~FORMATOPTION_quiet ) ) {
 			/* all options valid */
 			break;
 		};
 		
+		if ((input_is_pipe == 1) && ((formatoptions & FORMATOPTION_print_iid_var) == FORMATOPTION_print_iid_var)) {
+			/* workaround */
+			continue;
+		};
 		fprintf(stderr, " Unsupported format option(s):\n");
 
 		/* run through format options */
@@ -1000,6 +1123,7 @@ PIPE_input:
 		case FORMAT_ipv4revhex:
 		case FORMAT_base85:
 		case FORMAT_mac:
+		case FORMAT_eui64:
 		case FORMAT_ifinet6:
 		case FORMAT_revnibbles_int:
 		case FORMAT_revnibbles_arpa:
@@ -1007,7 +1131,22 @@ PIPE_input:
 			if (inputc < 1) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
 			break;
 
+		case FORMAT_iid:
+			if (inputc == 2) {
+				inputtype = FORMAT_iid_token;
+				if (outputtype == FORMAT_undefined) {
+					outputtype = FORMAT_iid_token;
+				};
+			};
+			break;
+
 		case FORMAT_iid_token:
+			if (inputc < 2) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
+			if (outputtype == FORMAT_undefined) {
+				outputtype = FORMAT_iid_token;
+			};
+			break;
+
 		case FORMAT_prefix_mac:
 			if (inputc < 2) { printhelp_missinginputdata(); exit(EXIT_FAILURE); };
 			break;
@@ -1054,6 +1193,11 @@ PIPE_input:
 			argc--;
 			break;
 
+		case FORMAT_eui64:
+			retval = libeui64_addr_to_eui64addrstruct(input1, resultstring, &eui64addr);
+			argc--;
+			break;
+
 		case FORMAT_ifinet6:
 			if (inputc < 2) {
 				retval = libifinet6_ifinet6_to_ipv6addrstruct(input1, resultstring, &ipv6addr);
@@ -1249,10 +1393,12 @@ PIPE_input:
 			outputtype = FORMAT_ipv6addr;
 		} else if ( (inputtype == FORMAT_ipv4addr) && (action == ACTION_anonymize) ) {
 			outputtype = FORMAT_ipv4addr;
-		} else if ( ((inputtype == FORMAT_ipv4addr) || (inputtype == FORMAT_ipv4hex) || (inputtype == FORMAT_ipv4revhex)) && (action == ACTION_undefined) ) {
+		} else if ( ((inputtype == FORMAT_ipv4addr) || (inputtype == FORMAT_ipv4hex) || (inputtype == FORMAT_ipv4revhex)) && (action == ACTION_undefined || action == ACTION_anonymize) ) {
 			outputtype = FORMAT_ipv4addr;
 		} else if ( (inputtype == FORMAT_mac) ) {
 			outputtype = FORMAT_mac;
+		} else if ( (inputtype == FORMAT_eui64) ) {
+			outputtype = FORMAT_eui64;
 		} else if ( (inputtype == FORMAT_bitstring) || (inputtype == FORMAT_base85) ) {
 			outputtype = FORMAT_ipv6addr;
 		} else if ( (inputtype == FORMAT_ipv4addr) && (action == ACTION_6rd_local_prefix) ) {
@@ -1320,20 +1466,27 @@ PIPE_input:
 			break;
 
 		case ACTION_anonymize:
-			if (inputtype == FORMAT_ipv4addr && outputtype == FORMAT_ipv4addr) {
+			if ((inputtype == FORMAT_ipv4addr || inputtype == FORMAT_ipv4hex || inputtype == FORMAT_ipv4revhex) && outputtype == FORMAT_ipv4addr) {
 				/* anonymize IPv4 address */
 				if (ipv4addr.flag_valid != 1) {
 					fprintf(stderr, "No valid IPv4 address given!\n");
 					exit(EXIT_FAILURE);
 				};
-				libipv4addr_anonymize(&ipv4addr, mask_ipv4);
-			} else if (inputtype == FORMAT_ipv6addr && outputtype == FORMAT_ipv6addr) {
+				libipv4addr_anonymize(&ipv4addr, ipv6calc_anon_set.mask_ipv4);
+			} else if ((inputtype == FORMAT_ipv6addr || inputtype == FORMAT_bitstring || inputtype == FORMAT_revnibbles_int || inputtype == FORMAT_revnibbles_arpa || inputtype == FORMAT_base85 || inputtype == FORMAT_ipv6literal) && outputtype == FORMAT_ipv6addr) {
 				/* anonymize IPv6 address */
 				if (ipv6addr.flag_valid != 1) {
 					fprintf(stderr, "No valid IPv6 address given!\n");
 					exit(EXIT_FAILURE);
 				};
-				libipv6addr_anonymize(&ipv6addr, mask_iid, mask_ipv6, mask_ipv4);
+				libipv6addr_anonymize(&ipv6addr, &ipv6calc_anon_set);
+			} else if (inputtype == FORMAT_mac && outputtype == FORMAT_mac) {
+				/* anonymize MAC address */
+				if (macaddr.flag_valid != 1) {
+					fprintf(stderr, "No valid MAC address given!\n");
+					exit(EXIT_FAILURE);
+				};
+				libmacaddr_anonymize(&macaddr, ipv6calc_anon_set.mask_mac);
 			} else {
 				fprintf(stderr, "Unsupported anonymization!\n");
 				exit(EXIT_FAILURE);
@@ -1428,6 +1581,40 @@ PIPE_input:
 			retval = librfc5569_calc_6rd_local_prefix(&ipv6addr, &ipv4addr2, &ipv4addr, resultstring);
 			break;
 
+		case ACTION_filter:
+			if (ipv6calc_debug != 0) {
+				fprintf(stderr, "%s: Start of action: filter\n", DEBUG_function_name);
+			};
+
+			outputtype = inputtype;
+
+			result = 1; /* default, skip */
+			if (inputtype == FORMAT_ipv4addr) {
+				/* call filter for IPv4 addresses */
+				result = ipv4addr_filter(&ipv4addr, &filter_master.filter_ipv4addr);
+			} else if (inputtype == FORMAT_ipv6addr) {
+				/* call filter for IPv6 addresses */
+				result = ipv6addr_filter(&ipv6addr, &filter_master.filter_ipv6addr);
+			} else if (inputtype == FORMAT_mac) {
+				/* call filter for MAC addresses */
+				result = macaddr_filter(&macaddr, &filter_master.filter_macaddr);
+			} else {
+				/* TODO: more specific notice */
+				fprintf(stderr, "Action-type isn't currently implemented for inputtype\n");
+			};
+
+			if (result != 0) {
+				/* skip everything */
+				if (ipv6calc_debug != 0) {
+					fprintf(stderr, "%s: filter result SKIP: '%s'\n", DEBUG_function_name, linebuffer);
+				};
+			} else {
+				snprintf(resultstring, sizeof(resultstring) - 1, "%s", linebuffer);
+			};
+			goto RESULT_print;
+
+			break;
+
 		case ACTION_undefined:
 			/* no action selected */
 			break;
@@ -1460,6 +1647,8 @@ PIPE_input:
 			retval = showinfo_ipv4addr(&ipv4addr, formatoptions);
 	       	} else if (macaddr.flag_valid == 1) {
 			retval = showinfo_eui48(&macaddr, formatoptions);
+	       	} else if (eui64addr.flag_valid == 1) {
+			retval = showinfo_eui64(&eui64addr, formatoptions);
 		} else {
 		       	fprintf(stderr, "No valid or supported input address given!\n");
 			retval = 1;
@@ -1549,6 +1738,19 @@ PIPE_input:
 			} else {
 				retval = librfc1884_ipv6addrstruct_to_compaddr(&ipv6addr, resultstring, formatoptions);
 			};
+
+			if ((formatoptions & FORMATOPTION_print_iid_var) == FORMATOPTION_print_iid_var) {
+				iid_privacy_result = ipv6addr_privacyextensiondetection(&ipv6addr, &iid_statistics);
+				sprintf(resultstring2, "%-40s R=%d h=%7.3f r=%7.3f db= %d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d da= %d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d dd= %d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d dda= %d", resultstring, \
+					iid_privacy_result, iid_statistics.hexdigit, iid_statistics.lls_residual, \
+					iid_statistics.digit_blocks[0], iid_statistics.digit_blocks[1], iid_statistics.digit_blocks[2], iid_statistics.digit_blocks[3], iid_statistics.digit_blocks[4], iid_statistics.digit_blocks[5], iid_statistics.digit_blocks[6], iid_statistics.digit_blocks[7], iid_statistics.digit_blocks[8], iid_statistics.digit_blocks[9], iid_statistics.digit_blocks[10], iid_statistics.digit_blocks[11], iid_statistics.digit_blocks[12], iid_statistics.digit_blocks[13], iid_statistics.digit_blocks[14], iid_statistics.digit_blocks[15], \
+					iid_statistics.digit_amount[0], iid_statistics.digit_amount[1], iid_statistics.digit_amount[2], iid_statistics.digit_amount[3], iid_statistics.digit_amount[4], iid_statistics.digit_amount[5], iid_statistics.digit_amount[6], iid_statistics.digit_amount[7], iid_statistics.digit_amount[8], iid_statistics.digit_amount[9], iid_statistics.digit_amount[10], iid_statistics.digit_amount[11], iid_statistics.digit_amount[12], iid_statistics.digit_amount[13], iid_statistics.digit_amount[14], iid_statistics.digit_amount[15], \
+					iid_statistics.digit_delta[0], iid_statistics.digit_delta[1], iid_statistics.digit_delta[2], iid_statistics.digit_delta[3], iid_statistics.digit_delta[4], iid_statistics.digit_delta[5], iid_statistics.digit_delta[6], iid_statistics.digit_delta[7], iid_statistics.digit_delta[8], iid_statistics.digit_delta[9], iid_statistics.digit_delta[10], iid_statistics.digit_delta[11], iid_statistics.digit_delta[12], iid_statistics.digit_delta[13], iid_statistics.digit_delta[14], iid_statistics.digit_delta[15], \
+					iid_statistics.digit_delta[16], iid_statistics.digit_delta[17], iid_statistics.digit_delta[18], iid_statistics.digit_delta[19], iid_statistics.digit_delta[20], iid_statistics.digit_delta[21], iid_statistics.digit_delta[22], iid_statistics.digit_delta[23], iid_statistics.digit_delta[24], iid_statistics.digit_delta[25], iid_statistics.digit_delta[26], iid_statistics.digit_delta[27], iid_statistics.digit_delta[28], iid_statistics.digit_delta[29], iid_statistics.digit_delta[30], \
+					iid_statistics.digit_delta_amount
+				);
+				sprintf(resultstring, "%s", resultstring2);
+			};
 			break;
 
 		case FORMAT_ipv6literal:
@@ -1609,7 +1811,7 @@ PIPE_input:
 
 RESULT_print:
 	/* print result */
-	if ( strlen(resultstring) > 0 ) {
+	if (strlen(resultstring) > 0) {
 		if ( retval == 0 ) {
 			fprintf(stdout, "%s\n", resultstring);
 		} else {
