@@ -1,8 +1,8 @@
 /*
  * Project    : ipv6calc
  * File       : showinfo.c
- * Version    : $Id: showinfo.c,v 1.54 2012/02/04 21:45:46 peter Exp $
- * Copyright  : 2001-2011 by Peter Bieringer <pb (at) bieringer.de>
+ * Version    : $Id: showinfo.c,v 1.70 2013/04/10 18:30:52 ds6peter Exp $
+ * Copyright  : 2001-2013 by Peter Bieringer <pb (at) bieringer.de>
  * 
  * Information:
  *  Function to show information about a given IPv6 address
@@ -22,7 +22,6 @@
 #include "libieee.h"
 #include "libmac.h"
 #include "libeui64.h"
-#include "libeui64.h"
 #include "config.h"
 #include "showinfo.h"
 
@@ -51,6 +50,11 @@ extern char file_geoip_ipv4[NI_MAXHOST];
 extern char file_geoip_ipv6[NI_MAXHOST];
 #endif
 
+/* from anonymizer */
+extern int mask_ipv6;
+extern int mask_ipv4;
+extern int mask_iid;
+extern s_ipv6calc_anon_set ipv6calc_anon_set;
 
 /*
  * show available types on machine readable format
@@ -59,21 +63,24 @@ void showinfo_availabletypes(void) {
 	int i;
 
 	fprintf(stderr, "\nAvailable tokens for machine-readable output (printed in one line):\n");
-	fprintf(stderr, " TYPE=...                      : type of IPv6 address (commata separated)\n");
+	fprintf(stderr, " IPV6_TYPE=...                 : type of IPv6 address (commata separated)\n");
 	fprintf(stderr, " ");
 	for (i = 0; i < (int) (sizeof(ipv6calc_ipv6addrtypestrings) / sizeof(ipv6calc_ipv6addrtypestrings[0])); i++ ) {
 		fprintf(stderr, " %s", ipv6calc_ipv6addrtypestrings[i].token);
 	};
 	fprintf(stderr, "\n");
 	fprintf(stderr, " IPV6=...                      : given IPv6 address full uncompressed\n");
+	fprintf(stderr, " IPV6_ANON=...                 : given anonymized IPv6 address full uncompressed\n");
 	fprintf(stderr, " IPV6_REGISTRY=...             : registry token of given IPv6 address\n");
 	fprintf(stderr, " IPV6_PREFIXLENGTH=ddd         : given prefix length\n");
 	fprintf(stderr, " IPV4=ddd.ddd.ddd.ddd          : native IPv4 address\n");
+	fprintf(stderr, " IPV4_ANON=ddd.ddd.ddd.ddd     : native anonymized IPv4 address\n");
 	fprintf(stderr, " IPV4_REGISTRY=...             : registry token of native IPv4 address\n");
 	fprintf(stderr, " IPV4_PREFIXLENGTH=ddd         : given prefix length of native IPv4 address\n");
-	fprintf(stderr, " IPV4[...]=ddd.ddd.ddd.ddd     : included IPv4 address in IID (e.g. ISATAP, TEREDO, NAT64)\n");
+	fprintf(stderr, " IPV4[...]=ddd.ddd.ddd.ddd     : included IPv4 address in IID or SLA (e.g. ISATAP, TEREDO, NAT64, 6to4)\n");
+	fprintf(stderr, " IPV4_ANON[...]=ddd.ddd.ddd.ddd: included anonymized IPv4 address in IID or SLA (e.g. ISATAP, TEREDO, NAT64, 6to4)\n");
 	fprintf(stderr, " IPV4_REGISTRY[...]=...        : registry token of included IPv4 address\n");
-	fprintf(stderr, " IPV4_SOURCE[...]=...           : source of IPv4 address\n");
+	fprintf(stderr, " IPV4_SOURCE[...]=...          : source of IPv4 address\n");
 	fprintf(stderr, "  ISATAP|TEREDO-SERVER|TEREDO-CLIENT|6TO4|LINK-LOCAL-IID\n");
 	fprintf(stderr, " SLA=xxxx                      : an included SLA\n");
 	fprintf(stderr, " IID=xxxx:xxxx:xxxx:xxxx       : an included interface identifier\n");
@@ -82,7 +89,7 @@ void showinfo_availabletypes(void) {
 	fprintf(stderr, " EUI48_TYPE=...                : type of EUI-48 identifier\n");
 	fprintf(stderr, "  unicast|multicast|broadcast\n");
 	fprintf(stderr, " EUI64=xx:xx:xx:xx:xx:xx:xx:xx : an included EUI-64 identifier\n");
-	fprintf(stderr, " EUI64_SCOPE=local|global      : scope of EUI-64 identifier\n");
+	fprintf(stderr, " EUI64_SCOPE=local-*|global    : scope of EUI-64 identifier\n");
 	fprintf(stderr, " OUI=\"...\"                     : OUI string, if available\n");
 	fprintf(stderr, " TEREDO_PORT_CLIENT=...        : port of Teredo client (NAT outside)\n");
 #ifdef SUPPORT_IP2LOCATION
@@ -114,6 +121,7 @@ void showinfo_availabletypes(void) {
 	fprintf(stderr, " IPV6CALC_COPYRIGHT=\"...\"      : Copyright string\n");
 	fprintf(stderr, " IPV6CALC_OUTPUT_VERSION=x     : Version of output format\n");
 	fprintf(stderr, " IPV6CALC_FEATURES=\"...\"       : Feature string of ipv6calc -v\n");
+	fprintf(stderr, " IPV6CALC_SETTINGS_ANON=...    : Anonymizer settings\n");
 };
 
 /*
@@ -142,6 +150,10 @@ static void printfooter(const uint32_t formatoptions) {
 		snprintf(tempstring, sizeof(tempstring) - 1, "IPV6CALC_OUTPUT_VERSION=%d", IPV6CALC_OUTPUT_VERSION);
 		printout(tempstring);
 
+		libipv6calc_anon_infostring(tempstring2, sizeof(tempstring2), &ipv6calc_anon_set);
+		snprintf(tempstring, sizeof(tempstring) - 1, "IPV6CALC_SETTINGS_ANON=%s", tempstring2);
+		printout(tempstring);
+
 		tempstring[0] = '\0'; /* clear tempstring */
 
 #ifdef SUPPORT_IP2LOCATION
@@ -476,12 +488,15 @@ static void print_geoip(const char *addrstring, const uint32_t formatoptions, co
 /* print IPv4 address */
 #define DEBUG_function_name "showinfo/print_ipv4addr"
 static void print_ipv4addr(const ipv6calc_ipv4addr *ipv4addrp, const uint32_t formatoptions, const char *string) {
-	char tempstring[NI_MAXHOST] = "", helpstring[NI_MAXHOST] = "";
+	char tempstring[NI_MAXHOST] = "", tempstring2[NI_MAXHOST] = "", helpstring[NI_MAXHOST] = "";
 	char tempipv4string[NI_MAXHOST] = "";
 	char embeddedipv4string[NI_MAXHOST] = "";
 	uint32_t machinereadable = (formatoptions & FORMATOPTION_machinereadable);
-	int retval;
+	int retval, i, j;
 	uint32_t typeinfo;
+	ipv6calc_ipv4addr ipv4addr_anon, *ipv4addr_anon_ptr;
+
+	ipv4addr_anon_ptr = &ipv4addr_anon;
 
 	typeinfo = ipv4addr_gettype(ipv4addrp);
 
@@ -497,6 +512,15 @@ static void print_ipv4addr(const ipv6calc_ipv4addr *ipv4addrp, const uint32_t fo
 	if ((formatoptions & FORMATOPTION_printembedded) != 0) {
 		snprintf(embeddedipv4string, sizeof(embeddedipv4string) - 1, "[%s]", tempipv4string);
 	};
+
+	ipv4addr_copy(ipv4addr_anon_ptr, ipv4addrp); /* copy structure */
+	libipv4addr_anonymize(ipv4addr_anon_ptr, ipv6calc_anon_set.mask_ipv4);
+	retval = libipv4addr_ipv4addrstruct_to_string(ipv4addr_anon_ptr, tempstring2, 0);
+	if ( retval != 0 ) {
+		fprintf(stderr, "Error uncompressing IPv4 address: %s\n", tempstring2);
+		retval = 1;
+		goto END_print_ipv4addr;
+	};	
 	
 	if ( machinereadable != 0 ) {
 		/* given source string */
@@ -509,17 +533,47 @@ static void print_ipv4addr(const ipv6calc_ipv4addr *ipv4addrp, const uint32_t fo
 		snprintf(tempstring, sizeof(tempstring) - 1, "IPV4%s=%s", embeddedipv4string, tempipv4string);
 		printout(tempstring);
 	
+		/* anonymized address */
+		snprintf(tempstring, sizeof(tempstring) - 1, "IPV4_ANON%s=%s", embeddedipv4string, tempstring2);
+		printout(tempstring);
+
 		if (ipv4addrp->flag_prefixuse == 1) {	
 			snprintf(tempstring, sizeof(tempstring) - 1, "IPV4_PREFIXLENGTH%s=%d", embeddedipv4string, (int) ipv4addrp->prefixlength);
 			printout(tempstring);
 		};
+
+		j = 0;
+		snprintf(tempstring, sizeof(tempstring) - 1, "IPV4_TYPE%s=", embeddedipv4string);
+		for (i = 0; i < (int) (sizeof(ipv6calc_ipv4addrtypestrings) / sizeof(ipv6calc_ipv4addrtypestrings[0])); i++ ) {
+			if ( (typeinfo & ipv6calc_ipv4addrtypestrings[i].number) != 0 ) {
+				if (j != 0) {
+					snprintf(helpstring, sizeof(helpstring) - 1, "%s,", tempstring);
+					snprintf(tempstring, sizeof(tempstring) - 1, "%s", helpstring);
+				};
+				snprintf(helpstring, sizeof(helpstring) - 1, "%s%s", tempstring, ipv6calc_ipv4addrtypestrings[i].token);
+				snprintf(tempstring, sizeof(tempstring) - 1, "%s", helpstring);
+				j = 1;
+			};
+		};
+		printout(tempstring);
 	} else {
 		fprintf(stdout, "IPv4 address: %s\n", tempipv4string);
+
+		fprintf(stdout, "IPv4 address type: ");
+		j = 0;
+		for (i = 0; i < (int) (sizeof(ipv6calc_ipv4addrtypestrings) / sizeof(ipv6calc_ipv4addrtypestrings[0])); i++ ) {
+			if ( (typeinfo & ipv6calc_ipv4addrtypestrings[i].number) != 0 ) {
+				if ( j != 0 ) { fprintf(stdout, ", "); };
+				fprintf(stdout, "%s", ipv6calc_ipv4addrtypestrings[i].token);
+				j = 1;
+			};
+		};
+		fprintf(stdout, "\n");
 	};	
 
 	/* get registry string */
 	retval = libipv4addr_get_registry_string(ipv4addrp, helpstring);
-	if ( retval != 0  && machinereadable == 0 ) {
+	if ( retval == 1  && machinereadable == 0 ) {
 		fprintf(stderr, "Error getting registry string for IPv4 address: %s (%s)\n", helpstring, tempipv4string);
 		return;
 	};
@@ -544,6 +598,7 @@ static void print_ipv4addr(const ipv6calc_ipv4addr *ipv4addrp, const uint32_t fo
 	};
 #endif
 
+END_print_ipv4addr:
 	return;
 };
 #undef DEBUG_function_name
@@ -702,32 +757,34 @@ static void print_eui64(const ipv6calc_eui64addr *eui64addrp, const uint32_t for
 #define DEBUG_function_name "showinfo_ipv6addr"
 int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t formatoptions) {
 	int retval = 1, i, j, flag_prefixuse, registry;
-	char tempstring[NI_MAXHOST] = "", helpstring[NI_MAXHOST] = "";
+	char tempstring[NI_MAXHOST] = "", tempstring2[NI_MAXHOST] = "", helpstring[NI_MAXHOST] = "";
 	char ipv6addrstring[NI_MAXHOST] = "";
-	ipv6calc_ipv6addr ipv6addr, *ipv6addrp;
+	ipv6calc_ipv6addr ipv6addr, ipv6addr_anon, *ipv6addrp, *ipv6addr_anon_ptr;
 	ipv6calc_ipv4addr ipv4addr, ipv4addr2;
 	ipv6calc_macaddr macaddr;
 	ipv6calc_eui64addr eui64addr;
 	uint16_t port;
 	uint32_t typeinfo;
 	uint32_t machinereadable = ( formatoptions & FORMATOPTION_machinereadable);
+	uint32_t payload;
+
+	ipv6addr_anon_ptr = &ipv6addr_anon;
 
 	ipv6addrp = &ipv6addr;
-	ipv6addr_copy(ipv6addrp, ipv6addrp1);
+	ipv6addr_copy(ipv6addrp, ipv6addrp1); /* copy structure */
 
 	typeinfo = ipv6addr_gettype(ipv6addrp);
 	registry = ipv6addr_getregistry(ipv6addrp);
 
 	if ( (ipv6calc_debug & DEBUG_showinfo) != 0) {
-		fprintf(stderr, "%s: result of 'ipv6addr_gettype'    : %x\n", DEBUG_function_name, (unsigned int) typeinfo);
 		fprintf(stderr, "%s: result of 'ipv6addr_getregistry': %d\n", DEBUG_function_name, registry);
+		fprintf(stderr, "%s:       %08x (result of 'ipv6addr_gettype')\n", DEBUG_function_name, (unsigned int) typeinfo);
 	};
 
 	for (i = 0; i < (int) (sizeof(ipv6calc_ipv6addrtypestrings) / sizeof(ipv6calc_ipv6addrtypestrings[0])); i++ ) {
 		if ( (ipv6calc_debug & DEBUG_showinfo) != 0) {
-			fprintf(stderr, "%s: test: %x : %s\n", DEBUG_function_name, (unsigned int) ipv6calc_ipv6addrtypestrings[i].number, ipv6calc_ipv6addrtypestrings[i].token);
+			fprintf(stderr, "%s: test: %08x : %s\n", DEBUG_function_name, (unsigned int) ipv6calc_ipv6addrtypestrings[i].number, ipv6calc_ipv6addrtypestrings[i].token);
 		};
-
 	};	
 
 	/* get full uncompressed IPv6 address */
@@ -741,18 +798,30 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 	};	
 
 	ipv6addrp->flag_prefixuse = flag_prefixuse;
+
+	ipv6addr_copy(ipv6addr_anon_ptr, ipv6addrp); /* copy structure */
+	libipv6addr_anonymize(ipv6addr_anon_ptr, &ipv6calc_anon_set);
+	retval = libipv6addr_ipv6addrstruct_to_uncompaddr(ipv6addr_anon_ptr, tempstring2, FORMATOPTION_printfulluncompressed);
+	if ( retval != 0 ) {
+		fprintf(stderr, "Error uncompressing IPv6 address: %s\n", tempstring2);
+		retval = 1;
+		goto END;
+	};	
 	
 	if ( machinereadable != 0 ) {
 		snprintf(tempstring, sizeof(tempstring) - 1, "IPV6=%s", ipv6addrstring);
 		printout(tempstring);
 	
+		snprintf(tempstring, sizeof(tempstring) - 1, "IPV6_ANON=%s", tempstring2);
+		printout(tempstring);
+
 		if (ipv6addrp->flag_prefixuse == 1) {	
 			snprintf(tempstring, sizeof(tempstring) - 1, "IPV6_PREFIXLENGTH=%d", (int) ipv6addrp->prefixlength);
 			printout(tempstring);
 		};
 
 		j = 0;
-		snprintf(tempstring, sizeof(tempstring) - 1, "TYPE=");
+		snprintf(tempstring, sizeof(tempstring) - 1, "IPV6_TYPE=");
 		for (i = 0; i < (int) (sizeof(ipv6calc_ipv6addrtypestrings) / sizeof(ipv6calc_ipv6addrtypestrings[0])); i++ ) {
 			if ( (typeinfo & ipv6calc_ipv6addrtypestrings[i].number) != 0 ) {
 				if (j != 0) {
@@ -873,7 +942,7 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 
 	/* get registry string */
 	retval = libipv6addr_get_registry_string(ipv6addrp, helpstring);
-	if ( retval != 0  && machinereadable == 0 ) {
+	if ( retval == 1  && machinereadable == 0 ) {
 		fprintf(stderr, "Error getting registry string for IPv6 address: %s\n", helpstring);
 	} else {
 		if ( machinereadable != 0 ) {
@@ -919,26 +988,65 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 			fprintf(stdout, "Interface identifier: %04x:%04x:%04x:%04x\n", (unsigned int) ipv6addr_getword(ipv6addrp, 4), (unsigned int) ipv6addr_getword(ipv6addrp, 5), (unsigned int) ipv6addr_getword(ipv6addrp, 6), (unsigned int) ipv6addr_getword(ipv6addrp, 7));
 		};
 
-		if (ipv6addr_getoctet(ipv6addrp, 11) == 0xff && ipv6addr_getoctet(ipv6addrp, 12) == 0xfe) {
+		if ((typeinfo & IPV6_ADDR_ANONYMIZED) == IPV6_ADDR_ANONYMIZED) {
+			if ( machinereadable != 0 ) {
+			} else {
+				fprintf(stdout, "Interface identifier is an anonymized one\n");
+			};
+		};
+
+		if ((typeinfo & IPV6_NEW_ADDR_IID_EUI48) == IPV6_NEW_ADDR_IID_EUI48) {
 			/* EUI-48 */
-			macaddr.addr[0] = ipv6addr_getoctet(ipv6addrp,  8) ^ 0x02;
-			macaddr.addr[1] = ipv6addr_getoctet(ipv6addrp,  9);
-			macaddr.addr[2] = ipv6addr_getoctet(ipv6addrp, 10);
-			macaddr.addr[3] = ipv6addr_getoctet(ipv6addrp, 13);
-			macaddr.addr[4] = ipv6addr_getoctet(ipv6addrp, 14);
-			macaddr.addr[5] = ipv6addr_getoctet(ipv6addrp, 15);
+			if ((typeinfo & IPV6_ADDR_ANONYMIZED) == IPV6_ADDR_ANONYMIZED) {
+				payload = ipv6addr_get_payload_anonymized_iid(ipv6addrp, typeinfo) ^ 0x020000;
+
+				if ((payload & 0x1000000) != 0) {
+					libieee_unmap_oui_macaddr(&macaddr, payload);
+				} else {
+					macaddr.addr[0] = ((payload >> 16) & 0xff) ^ 0x02;
+					macaddr.addr[1] = (payload >> 8) & 0xff;
+					macaddr.addr[2] = (payload & 0xff);
+					macaddr.addr[3] = 0;
+					macaddr.addr[4] = 0;
+					macaddr.addr[5] = 0;
+				};
+			} else {
+				macaddr.addr[0] = ipv6addr_getoctet(ipv6addrp,  8) ^ 0x02;
+				macaddr.addr[1] = ipv6addr_getoctet(ipv6addrp,  9);
+				macaddr.addr[2] = ipv6addr_getoctet(ipv6addrp, 10);
+				macaddr.addr[3] = ipv6addr_getoctet(ipv6addrp, 13);
+				macaddr.addr[4] = ipv6addr_getoctet(ipv6addrp, 14);
+				macaddr.addr[5] = ipv6addr_getoctet(ipv6addrp, 15);
+			};
 			print_eui48(&macaddr, formatoptions);
 		} else {
-			/* Check for global EUI-64 */
-			if ( (ipv6addr_getoctet(ipv6addrp, 8) & 0x02) != 0 ) {
-				eui64addr.addr[0] = ipv6addr_getoctet(ipv6addrp,  8) ^ 0x02;
-				eui64addr.addr[1] = ipv6addr_getoctet(ipv6addrp,  9);
-				eui64addr.addr[2] = ipv6addr_getoctet(ipv6addrp, 10);
-				eui64addr.addr[3] = ipv6addr_getoctet(ipv6addrp, 11);
-				eui64addr.addr[4] = ipv6addr_getoctet(ipv6addrp, 12);
-				eui64addr.addr[5] = ipv6addr_getoctet(ipv6addrp, 13);
-				eui64addr.addr[6] = ipv6addr_getoctet(ipv6addrp, 14);
-				eui64addr.addr[7] = ipv6addr_getoctet(ipv6addrp, 15);
+			if ((typeinfo & IPV6_NEW_ADDR_IID_EUI64) == IPV6_NEW_ADDR_IID_EUI64) {
+				/* EUI-64 */
+				if ((typeinfo & IPV6_ADDR_ANONYMIZED) == IPV6_ADDR_ANONYMIZED) {
+					payload = ipv6addr_get_payload_anonymized_iid(ipv6addrp, typeinfo) ^ 0x020000;
+
+					if ((payload & 0x1000000) != 0) {
+						libieee_unmap_oui_eui64addr(&eui64addr, payload);
+					} else  {
+						eui64addr.addr[0] = ((payload >> 16) & 0xff) ^ 0x02;
+						eui64addr.addr[1] = (payload >> 8) & 0xff;
+						eui64addr.addr[2] = (payload & 0xff);
+						eui64addr.addr[3] = 0;
+						eui64addr.addr[4] = 0;
+						eui64addr.addr[5] = 0;
+						eui64addr.addr[6] = 0;
+						eui64addr.addr[7] = 0;
+					};
+				} else {
+					eui64addr.addr[0] = ipv6addr_getoctet(ipv6addrp,  8) ^ 0x02;
+					eui64addr.addr[1] = ipv6addr_getoctet(ipv6addrp,  9);
+					eui64addr.addr[2] = ipv6addr_getoctet(ipv6addrp, 10);
+					eui64addr.addr[3] = ipv6addr_getoctet(ipv6addrp, 11);
+					eui64addr.addr[4] = ipv6addr_getoctet(ipv6addrp, 12);
+					eui64addr.addr[5] = ipv6addr_getoctet(ipv6addrp, 13);
+					eui64addr.addr[6] = ipv6addr_getoctet(ipv6addrp, 14);
+					eui64addr.addr[7] = ipv6addr_getoctet(ipv6addrp, 15);
+				};
 				print_eui64(&eui64addr, formatoptions);
 			} else {
 				if ( (typeinfo & IPV6_NEW_ADDR_SOLICITED_NODE) != 0 ) {
@@ -948,16 +1056,42 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 					} else {
 						fprintf(stdout, "Generated from the extension identifier of an EUI-48 (MAC): ...:%02x:%02x:%02x\n", (unsigned int) ipv6addr_getoctet(ipv6addrp, 13), (unsigned int) ipv6addr_getoctet(ipv6addrp, 14), (unsigned int) ipv6addr_getoctet(ipv6addrp, 15));
 					};
-				} else if ( (typeinfo & IPV6_NEW_ADDR_ISATAP) != 0 )  {
-					for (i = 0; i <= 3; i++) {
-						ipv4addr_setoctet(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctet(ipv6addrp, (unsigned int) (i + 12)));
-					};
+				} else if ( (typeinfo & (IPV6_NEW_ADDR_IID_ISATAP | IPV6_NEW_ADDR_6TO4_MICROSOFT) ) != 0 )  {
+					if ((typeinfo & IPV6_ADDR_IID_32_63_HAS_IPV4) != 0) {
+						/* IPv4 address included */
 
-					if ( machinereadable != 0 ) {
+						if ((typeinfo & IPV6_ADDR_ANONYMIZED) != 0) {
+							payload = ipv6addr_get_payload_anonymized_iid(ipv6addrp, typeinfo);
+							/* IPv4 */
+							ipv4addr_setoctet(&ipv4addr, 0, (payload >> 16) & 0xff);
+							ipv4addr_setoctet(&ipv4addr, 1, (payload >>  8) & 0xff);
+							ipv4addr_setoctet(&ipv4addr, 2, (payload      ) & 0xff);
+							ipv4addr_setoctet(&ipv4addr, 3, 0);
+						} else {
+							for (i = 0; i <= 3; i++) {
+								ipv4addr_setoctet(&ipv4addr, (unsigned int) i, (unsigned int) ipv6addr_getoctet(ipv6addrp, (unsigned int) (i + 12)));
+							};
+						};
+
+						if ( (typeinfo & IPV6_NEW_ADDR_IID_ISATAP) != 0 )  {
+							print_ipv4addr(&ipv4addr, formatoptions | FORMATOPTION_printembedded, "ISATAP");
+						} else {
+							print_ipv4addr(&ipv4addr, formatoptions | FORMATOPTION_printembedded, "6to4-microsoft");
+						};
+
+					} else if ((ipv6addr_getoctet(ipv6addrp, 11) == 0xff) && (ipv6addr_getoctet(ipv6addrp, 12) == 0xfe)) {
+						/* Vendor ID included */
+						if ( machinereadable != 0 ) {
+						} else {
+							fprintf(stdout, "ISATAP vendor ID: 0x%02x%02x%02x\n", ipv6addr_getoctet(ipv6addrp, 13), ipv6addr_getoctet(ipv6addrp, 14), ipv6addr_getoctet(ipv6addrp, 15));
+						};
 					} else {
-						fprintf(stdout, "IPv4 registry for ISATAP client address: %s\n", helpstring);
+						/* Extension ID included */
+						if ( machinereadable != 0 ) {
+						} else {
+							fprintf(stdout, "ISATAP extension ID: 0x%02x%02x%02x%02x%02x\n", ipv6addr_getoctet(ipv6addrp, 11), ipv6addr_getoctet(ipv6addrp, 12), ipv6addr_getoctet(ipv6addrp, 13), ipv6addr_getoctet(ipv6addrp, 14), ipv6addr_getoctet(ipv6addrp, 15));
+						};
 					};
-					print_ipv4addr(&ipv4addr, formatoptions | FORMATOPTION_printembedded, "ISATAP");
 				} else if ( ( ( (typeinfo & IPV6_ADDR_LINKLOCAL) != 0) && (ipv6addr_getdword(ipv6addrp, 2) == 0 && ipv6addr_getword(ipv6addrp, 6) != 0)) )   {
 					/* fe80:: must have 0000:0000:xxxx:yyyy where xxxx > 0 */
 					if ( machinereadable != 0 ) {
@@ -975,14 +1109,18 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 					if ( machinereadable != 0 ) {
 						if ((typeinfo & IPV6_NEW_ADDR_6TO4_MICROSOFT) != 0) {
 							printout("EUI64_SCOPE=local-6to4-microsoft");
+						} else if ((typeinfo & IPV6_NEW_ADDR_IID_PRIVACY) != 0) {
+							printout("EUI64_SCOPE=local-privacy");
 						} else {
 							printout("EUI64_SCOPE=local");
 						};
 					} else {
 						if ((typeinfo & IPV6_NEW_ADDR_6TO4_MICROSOFT) != 0) {
 							fprintf(stdout, "Interface identifier contain only IPv4 address from 6to4 prefix, usually seen on Microsoft OS\n");
+						} else if ((typeinfo & IPV6_NEW_ADDR_IID_PRIVACY) != 0) {
+							fprintf(stdout, "Interface identifier is probably generated by privacy extension\n");
 						} else {
-							fprintf(stdout, "Interface identifier is probably manual set or based on a local EUI-64 identifier\n");
+							fprintf(stdout, "Interface identifier is probably manual set\n");
 						};
 					};
 				};
@@ -999,7 +1137,11 @@ int showinfo_ipv6addr(const ipv6calc_ipv6addr *ipv6addrp1, const uint32_t format
 
 		if ( machinereadable != 0 ) {
 		} else {
-			snprintf(tempstring, sizeof(tempstring) - 1, "%s", ipv6addrstring+7);
+			if ((typeinfo & IPV6_ADDR_ANONYMIZED) != 0) {
+				snprintf(tempstring, sizeof(tempstring) - 1, "n/a (previosly anonymized)");
+			} else {
+				snprintf(tempstring, sizeof(tempstring) - 1, "%s", ipv6addrstring+7);
+			};
 			fprintf(stdout, "ORCHID hash (100 bits): %s\n", tempstring);
 		};
 	};
@@ -1064,3 +1206,20 @@ int showinfo_eui48(const ipv6calc_macaddr *macaddrp, const uint32_t formatoption
 	return (retval);
 };
 #undef DEBUG_function_name
+
+
+/*
+ * shows information about a given EUI-64 identifier
+ *
+ * in : *eui64addrp = pointer to EUI-64 address
+ * ret: ==0: ok, !=0: error
+ */
+int showinfo_eui64(const ipv6calc_eui64addr *eui64addrp, const uint32_t formatoptions) {
+	int retval = 1;
+
+	print_eui64(eui64addrp, formatoptions);
+	printfooter(formatoptions);
+
+	retval = 0;
+	return (retval);
+};
