@@ -2,8 +2,8 @@
 #
 # Project    : ipv6calc
 # File       : test_ipv6calc.sh
-# Version    : $Id: test_ipv6calc.sh,v 1.30 2012/01/10 20:50:16 peter Exp $
-# Copyright  : 2001-2011 by Peter Bieringer <pb (at) bieringer.de>
+# Version    : $Id: test_ipv6calc.sh,v 1.41 2013/05/10 05:51:04 ds6peter Exp $
+# Copyright  : 2001-2013 by Peter Bieringer <pb (at) bieringer.de>
 #
 # Test patterns for ipv6calc conversions
 
@@ -49,6 +49,7 @@ cat <<END | grep -v '^#'
 --addr_to_compressed 0:0:0:0:0:0:13.1.68.3				=::13.1.68.3
 --addr_to_compressed 0:0:0:0:0:ffff:129.144.52.38			=::ffff:129.144.52.38
 --addr_to_compressed --uppercase 0:0:0:0:0:ffff:129.144.52.38		=::FFFF:129.144.52.38
+--addr_to_compressed fd00:1234:5678:0:210:a4ff:fe01:2345		=fd00:1234:5678::210:a4ff:fe01:2345
 --in ipv6 --out ipv6 --printcompressed --uppercase 0:0:0:0:0:ffff:129.144.52.38		=::FFFF:129.144.52.38
 ## uncompressed
 --addr_to_uncompressed 3ffe:ffff:100:f101::1				=3ffe:ffff:100:f101:0:0:0:1
@@ -105,8 +106,8 @@ cat <<END | grep -v '^#'
 ## MAC to EUI-64 (autodetect mode, not supported in pipe mode)
 NOPIPETEST--out eui64 00:0:F:6:4:5					=200:fff:fe06:405
 ## Interface identifier privacy conversion
---eui64_to_privacy 0123:4567:89ab:cdef 0123456789abcdef			=4662:bdea:8654:776d 486072ff7074945e
---in iid+token --out iid+token 0123:4567:89ab:cdef 0123456789abcdef			=4662:bdea:8654:776d 486072ff7074945e
+--eui64_to_privacy 0123:4567:89ab:cdef 0123456789abcdef			=4462:bdea:8654:776d 486072ff7074945e
+--in iid+token --out iid+token 0123:4567:89ab:cdef 0123456789abcdef			=4462:bdea:8654:776d 486072ff7074945e
 ## Bitstring
 --addr_to_bitstring 3ffe:ffff::1					=\\\\[x3ffeffff000000000000000000000001/128].ip6.arpa.
 --in ipv6 --out bitstring 3ffe:ffff::1				=\\\\[x3ffeffff000000000000000000000001/128].ip6.arpa.
@@ -129,14 +130,35 @@ NOPIPETEST--out eui64 00:0:F:6:4:5					=200:fff:fe06:405
 -i fe80::1								=*
 -i -m ff02::1								=*
 # Anonymization
+--action anonymize fe80:0000:0000:0000:0200:5efe:192.0.2.143		=fe80::a909:5214:5c00:21
+--action anonymize fe80:0000:0000:0000:0000:5e01:2345:6789		=fe80::a909:5214:e012:3451
+--action anonymize fe80:0000:0000:0000:0200:5e01:2345:6789		=fe80::a909:5214:f012:345f
+--action anonymize fe80:0000:0000:0000:0000:5eff:fe01:2345		=fe80::a909:5214:c010:1
+--action anonymize 2001:0db8:0000:0000:81c0:0f3f:c807:1455		=2001:db8::9:a929:4941:0:c
 --action anonymize 3ffe:831f:ce49:7601:8000:efff:af4a:86BF		=3ffe:831f:ce49:7601:8000:ffff:af4a:86ff
 --action anonymize --mask-ipv4 16 3ffe:831f:ce49:7601:8000:efff:af4a:86BF	=3ffe:831f:ce49:7601:8000:ffff:af4a:ffff
 --action anonymize 192.0.2.1						=192.0.2.0
 --action anonymize --mask-ipv4 16 192.0.2.1				=192.0.0.0
+--action anonymize 2001:1a:392e:a450:2cd3:75e1:6098:8104		=2001:19:a909:a909:a999:4843::e
+--action anonymize 01:23:45:67:89:ab					=01:23:45:00:00:00
+--action anonymize --mask-mac 48 01:23:45:67:89:ab			=01:23:45:67:89:ab
+--action anonymize --mask-mac 47 01:23:45:67:89:ab			=01:23:45:67:89:aa
+--action anonymize --mask-mac 44 01:23:45:67:89:ab			=01:23:45:67:89:a0
+--action anonymize --mask-mac 41 01:23:45:67:89:a8			=01:23:45:67:89:80
+--action anonymize --mask-mac 40 01:23:45:67:89:a8			=01:23:45:67:89:00
+--action anonymize --mask-mac 8  ff:23:45:67:89:a8			=ff:00:00:00:00:00
+--action anonymize --mask-mac 7  ff:23:45:67:89:a8			=fe:00:00:00:00:00
+--action anonymize --mask-mac 1  ff:23:45:67:89:a8			=80:00:00:00:00:00
+-A anonymize --anonymize-preset zeroize-paranoid   2001:db8:2280:6901:224:21ff:fe01:2345  =2001:db8:2200::224:21ff:fe00:0
+-A anonymize --anonymize-preset zeroize-careful    2001:db8:2280:6901:224:21ff:fe01:2345  =2001:db8:2280::224:21ff:fe00:0
+-A anonymize --anonymize-preset zeroize-standard   2001:db8:2280:6901:224:21ff:fe01:2345  =2001:db8:2280:6900:224:21ff:fe00:0
+-A anonymize --anonymize-preset anonymize-paranoid 2001:db8:2280:6901:224:21ff:fe01:2345  =2001:db8:2209:a909:a969:4291:4022:4213
+-A anonymize --anonymize-preset anonymize-careful  2001:db8:2280:6901:224:21ff:fe01:2345  =2001:db8:2280:a909:a949:4291:4022:4219
+-A anonymize --anonymize-preset anonymize-standard 2001:db8:2280:6901:224:21ff:fe01:2345  =2001:db8:2280:6909:a929:4291:4022:4217
 # RFC 5952 4.2.1
 --in ipv6addr --out ipv6addr 2001:db8:0:0:0:0:2:1			=2001:db8::2:1
 # RFC 5952 4.2.2
---in ipv6addr --out ipv6addr 2001:db8:0:1:1:1:1:1			=2001:db8:0:1:1:1:1:1
+--in ipv6addr --out ipv6addr 2001:db8:0:1:1:1:1:1			=2001:db8::1:1:1:1:1
 # RFC 5952 4.2.3
 --in ipv6addr --out ipv6addr 2001:0:0:1:0:0:0:1			 	=2001:0:0:1::1
 --in ipv6addr --out ipv6addr 2001:db8:0:0:1:0:0:1 			=2001:db8::1:0:0:1
@@ -234,7 +256,7 @@ testscenarios | sed 's/NOPIPETEST//' | while read line; do
 	# Check result
 	if [ "$result" != "*" ]; then
 		if [ "$output" != "$result" ]; then
-			echo "Result '$output' doesn't match!"
+			echo "Result '$output' doesn't match: $result"
 			exit 1
 		fi
 	fi
@@ -335,7 +357,7 @@ done || exit 1
 
 echo "Run 'ipv6calc' pipe tests (1)"
 testscenarios_pipe | while IFS="," read input arguments result; do
-	echo "Test 'echo $input | ./ipv6calc $arguments | grep \"^$result\$i\"'"
+	echo "Test 'echo $input | ./ipv6calc $arguments | grep \"^$result\$\"'"
 	#set -x
 	echo -e $input | ./ipv6calc $arguments | grep "^$result\$" >/dev/null
 	retval=$?
@@ -367,7 +389,7 @@ testscenarios | grep -v "^NOPIPETEST" | while read line; do
 		options="`echo "$command" | awk '{ for (i=1; i < NF-1; i++) printf "%s ", $i; }'`"
 		;;
 	esac
-	echo "Test 'echo $stdin | ./ipv6calc $options | grep \"^$result\$i\"'"
+	echo "Test 'echo $stdin | ./ipv6calc $options | grep \"^$result\$\"'"
 	#set -x
 	output="`echo -e $stdin | ./ipv6calc $options`"
 	retval=$?
@@ -402,7 +424,6 @@ echo "Run 'ipv6calc' pipe mode input validation tests...(too long input)"
 	fi
 done || exit 1
 
-
 retval=$?
 if [ $retval -eq 0 ]; then
 	echo "All tests were successfully done!"
@@ -411,3 +432,4 @@ else
 	echo "Tests failed! (code $retval)"
 	exit $retval
 fi
+
