@@ -1,8 +1,8 @@
 /*
  * Project    : ipv6calc/ipv6logstats
  * File       : ipv6logstatsoptions.h
- * Version    : $Id: ipv6logstatsoptions.h,v 1.3 2003/11/22 12:42:44 peter Exp $
- * Copyright  : 2003 by Peter Bieringer <pb (at) bieringer.de>
+ * Version    : $Id: ipv6logstatsoptions.h,v 1.6 2012/05/09 17:08:10 peter Exp $
+ * Copyright  : 2003-2012 by Peter Bieringer <pb (at) bieringer.de>
  *
  * Information:
  *  Header file containing optios for ipvlogstats.c
@@ -20,7 +20,7 @@
 /* Options */
 
 /* define short options */
-static char *ipv6logstats_shortopts = "vqh?uoncd:p:";
+static char *ipv6logstats_shortopts = "vqh?uoncd:p:w:";
 
 /* define long options */
 static struct option ipv6logstats_longopts[] = {
@@ -37,6 +37,7 @@ static struct option ipv6logstats_longopts[] = {
 	{"prefix", 0, 0, (int) 'p'},
 	{"noheader", 0, 0, (int) 'n'},
 	{"onlyheader", 0, 0, (int) 'o'},
+	{"write", 1, 0, (int) 'O'},
 
 	{NULL, 0, 0, 0}
 };                
