@@ -1,8 +1,8 @@
 /*
  * Project    : ipv6calc
  * File       : ipv6calchelp.c
- * Version    : $Id: ipv6calchelp.c,v 1.26 2011/11/27 15:44:41 peter Exp $
- * Copyright  : 2002-2011 by Peter Bieringer <pb (at) bieringer.de>
+ * Version    : $Id: ipv6calchelp.c,v 1.31 2013/05/12 07:23:12 ds6peter Exp $
+ * Copyright  : 2002-2013 by Peter Bieringer <pb (at) bieringer.de>
  *
  * Information:
  *  Help library
@@ -313,9 +313,9 @@ void printhelp(void) {
 	fprintf(stderr, "  [<format option> ...] : specify format options\n");
 	fprintf(stderr, "  <input data> [...]    : input data\n");
 	fprintf(stderr, "\n");
-	fprintf(stderr, "  Available input  types:  [-m] --in     -?|-h|--help\n");
-	fprintf(stderr, "  Available output types:  [-m] --out    -?|-h|--help\n");
-	fprintf(stderr, "  Available action types:  [-m] --action -?|-h|--help\n");
+	fprintf(stderr, "  Available input  types:  [-m] -I|--in     -?|-h|--help\n");
+	fprintf(stderr, "  Available output types:  [-m] -O|--out    -?|-h|--help\n");
+	fprintf(stderr, "  Available action types:  [-m] -A|--action -?|-h|--help\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, " Other usage:\n");
 	fprintf(stderr, "\n");
@@ -582,3 +582,109 @@ void printhelp_output_dispatcher(const uint32_t outputtype) {
 	};
 	fprintf(stderr, "\n");
 };
+
+/* help regarding action command */
+/* in: embedded = 1 : do not show version and command */
+void printhelp_action_dispatcher(const uint32_t action, const int embedded) {
+	int i, j;
+	char method_name[32];
+
+	if (embedded != 1) {
+		printversion();
+	};
+	
+	fprintf(stderr, "\n");
+
+	switch (action) {
+		case ACTION_mac_to_eui64:
+			fprintf(stderr, " help still missing - sorry.\n");
+			break;
+			
+		case ACTION_ipv4_to_6to4addr:
+			fprintf(stderr, " help still missing - sorry.\n");
+			break;
+
+		case ACTION_anonymize:
+			if (embedded != 1) {
+				fprintf(stderr, " Anonymize given address according to preset or custom values, e.g.\n");
+				fprintf(stderr, "  ipv6calc -A anonymize 2001:db8:2280:6901:224:21ff:fe01:2345 --anonymize-preset zeroize-standard\n");
+				fprintf(stderr, "   2001:db8:2280:6900:224:21ff:fe00:0\n");
+				fprintf(stderr, "  ipv6calc -A anonymize 2001:db8:2280:6901:224:21ff:fe01:2345 --anonymize-preset anonymize-standard\n");
+				fprintf(stderr, "   2001:db8:2280:6909:a929:4291:4022:4217\n");
+				fprintf(stderr, "\n");
+			};
+
+			fprintf(stderr, "  Shortcut for anonymization presets:\n");
+			fprintf(stderr, "   --anonymize-standard (default)\n");
+			fprintf(stderr, "   --anonymize-careful\n");
+			fprintf(stderr, "   --anonymize-paranoid\n");
+			fprintf(stderr, "\n");
+			fprintf(stderr, "  Supported methods [--anonymize-method METHOD]:\n");
+			for (i = 0; i < sizeof(ipv6calc_anon_methods) / sizeof(s_ipv6calc_anon_methods); i++) {
+				fprintf(stderr, "   %-10s: %s\n", ipv6calc_anon_methods[i].name, ipv6calc_anon_methods[i].description);
+			};
+			fprintf(stderr, "\n");
+
+			fprintf(stderr, "  Available presets (shortcut names) [--anonymize-preset PRESET-NAME]:\n");
+
+			for (i = 0; i < sizeof(ipv6calc_anon_set_list) / sizeof(s_ipv6calc_anon_set); i++) {
+				snprintf(method_name, sizeof(method_name) - 1, "%s", "unknown"); // default
+
+				for (j = 0; j < sizeof(ipv6calc_anon_methods) / sizeof(s_ipv6calc_anon_methods); j++) {
+					if (ipv6calc_anon_methods[j].method == ipv6calc_anon_set_list[i].method) {
+						snprintf(method_name, sizeof(method_name) - 1, "%s", ipv6calc_anon_methods[j].name);
+						break;
+					};
+				};
+
+				fprintf(stderr, "   %-20s (%2s): mask-ipv6=%3d mask-ipv4=%2d mask-iid=%2d mask-mac=%2d method=%s\n", ipv6calc_anon_set_list[i].name, ipv6calc_anon_set_list[i].name_short, ipv6calc_anon_set_list[i].mask_ipv6, ipv6calc_anon_set_list[i].mask_ipv4, ipv6calc_anon_set_list[i].mask_iid, ipv6calc_anon_set_list[i].mask_mac, method_name);
+			};
+			fprintf(stderr, "\n");
+
+			fprintf(stderr, "  Custom control:\n");
+			fprintf(stderr, "  --mask-ipv4 <bits>     : mask IPv4 address [0-32] (even if occurs in IPv6 address)\n");
+			fprintf(stderr, "  --mask-ipv6 <bits>     : mask IPv6 prefix [0-64] (only applied to related address types)\n");
+			fprintf(stderr, "  --mask-iid  <bits>     : mask IPv6 interface identifiers [0-64]\n");
+			fprintf(stderr, "  --mask-mac  <bits>     : mask MAC address [0-48]\n");
+
+			break;
+			
+		case ACTION_iid_token_to_privacy:
+			fprintf(stderr, " help still missing - sorry.\n");
+			break;
+
+		case ACTION_prefix_mac_to_ipv6:
+			fprintf(stderr, " help still missing - sorry.\n");
+			break;
+
+		case ACTION_6rd_local_prefix:
+			fprintf(stderr, " help still missing - sorry.\n");
+			break;
+
+		case ACTION_filter:
+			fprintf(stderr, " Filter given addresses from stdout by filter expression, e.g.\n");
+			fprintf(stderr, "  echo '2001:db8::1' | ipv6calc -A filter -E iid-local\n");
+			fprintf(stderr, "  echo '2001:db8::1' | ipv6calc -A filter -E iid-local,global-unicast\n");
+			fprintf(stderr, "  echo '2001:db8::1' | ipv6calc -A filter -E ^iid-privacy\n");
+			fprintf(stderr, "\n");
+			fprintf(stderr, "  IPv6 address filter tokens:\n");
+			fprintf(stderr, "   ipv6 ");
+			for (i = 0; i < (int) (sizeof(ipv6calc_ipv6addrtypestrings) / sizeof(ipv6calc_ipv6addrtypestrings[0])); i++ ) {
+				fprintf(stderr, " %s", ipv6calc_ipv6addrtypestrings[i].token);
+			};
+			fprintf(stderr, "\n");
+			fprintf(stderr, "\n");
+			fprintf(stderr, "  IPv4 address filter tokens:\n");
+			fprintf(stderr, "   ipv4 ");
+			for (i = 0; i < (int) (sizeof(ipv6calc_ipv4addrtypestrings) / sizeof(ipv6calc_ipv4addrtypestrings[0])); i++ ) {
+				fprintf(stderr, " %s", ipv6calc_ipv4addrtypestrings[i].token);
+			};
+			fprintf(stderr, "\n");
+			fprintf(stderr, "\n");
+			fprintf(stderr, "  MAC address filter tokens:\n");
+			fprintf(stderr, "   ");
+			fprintf(stderr, " IMPLEMENTATION MISSING");
+			fprintf(stderr, "\n");
+			break;
+	};
+};
