@@ -1,8 +1,8 @@
 /*
  * Project    : ipv6calc
  * File       : ipv6calctypes.h
- * Version    : $Id: ipv6calctypes.h,v 1.26 2012/01/10 20:50:16 peter Exp $
- * Copyright  : 2002-2012 by Peter Bieringer <pb (at) bieringer.de>
+ * Version    : $Id: ipv6calctypes.h,v 1.31 2013/03/30 18:03:45 ds6peter Exp $
+ * Copyright  : 2002-2013 by Peter Bieringer <pb (at) bieringer.de>
  *
  * Information:
  *  Header file for ipv6calctypes.c
@@ -22,6 +22,13 @@ extern uint32_t ipv6calctypes_checkaction(const char *string);
 
 #define _ipv6calctypes_h_
 
+/* text representations for defines */
+typedef struct {
+	const uint32_t number;
+	const char *token;
+} s_type;
+
+
 /*
  * Format handling matrix for new address option style
  */
@@ -63,6 +70,7 @@ extern uint32_t ipv6calctypes_checkaction(const char *string);
 #define FORMAT_ipv6addr		(uint32_t) (1 << FORMAT_NUM_ipv6addr)
 #define FORMAT_ipv4addr		(uint32_t) (1 << FORMAT_NUM_ipv4addr)
 #define FORMAT_mac		(uint32_t) (1 << FORMAT_NUM_mac)
+#define FORMAT_macaddr		(uint32_t) (1 << FORMAT_NUM_mac)	// compatible syntax
 #define FORMAT_eui64		(uint32_t) (1 << FORMAT_NUM_eui64)
 #define FORMAT_base85		(uint32_t) (1 << FORMAT_NUM_base85)
 #define FORMAT_ifinet6		(uint32_t) (1 << FORMAT_NUM_ifinet6)
@@ -158,6 +166,7 @@ typedef struct {
 #define FORMATOPTION_NUM_printmirrored		14	// 00004000
 #define FORMATOPTION_NUM_forceprefix		15	// 00010000
 #define FORMATOPTION_NUM_literal		16	// 00020000
+#define FORMATOPTION_NUM_print_iid_var		17	// 00040000
 
 #define FORMATOPTION_NUM_printembedded		31	// special
 
@@ -177,6 +186,7 @@ typedef struct {
 #define FORMATOPTION_printmirrored		(uint32_t) (1 << FORMATOPTION_NUM_printmirrored)
 #define FORMATOPTION_forceprefix		(uint32_t) (1 << FORMATOPTION_NUM_forceprefix)
 #define FORMATOPTION_literal			(uint32_t) (1 << FORMATOPTION_NUM_literal)
+#define FORMATOPTION_print_iid_var		(uint32_t) (1 << FORMATOPTION_NUM_print_iid_var)
 
 #define FORMATOPTION_printembedded		(uint32_t) (1 << FORMATOPTION_NUM_printembedded)
 
@@ -202,6 +212,7 @@ typedef struct {
 	{ FORMATOPTION_quiet            , "--quiet|-q"         , "Be more quiet" },
 	{ FORMATOPTION_printmirrored	, "--printmirrored"    , "Print output mirrored" },
 	{ FORMATOPTION_forceprefix	, "--forceprefix <num>", "Force prefix to <num>" },
+	{ FORMATOPTION_print_iid_var	, "--print-iid-var"	, "IPv6 address IID variance"},
 };
 
 /* Possible format option map */
@@ -210,7 +221,7 @@ typedef struct {
 	{ FORMAT_revnibbles_arpa, FORMATOPTION_printlowercase | FORMATOPTION_printuppercase | FORMATOPTION_printprefix | FORMATOPTION_printsuffix | FORMATOPTION_maskprefix | FORMATOPTION_masksuffix | FORMATOPTION_printstart | FORMATOPTION_printend | FORMATOPTION_printmirrored },
 	{ FORMAT_revipv4, FORMATOPTION_printlowercase | FORMATOPTION_printuppercase | FORMATOPTION_printmirrored },
 	{ FORMAT_bitstring      , FORMATOPTION_printlowercase | FORMATOPTION_printuppercase | FORMATOPTION_printprefix | FORMATOPTION_printsuffix | FORMATOPTION_maskprefix | FORMATOPTION_masksuffix | FORMATOPTION_printstart | FORMATOPTION_printend },
-	{ FORMAT_ipv6addr       , FORMATOPTION_machinereadable | FORMATOPTION_printlowercase | FORMATOPTION_printuppercase | FORMATOPTION_printprefix | FORMATOPTION_printsuffix | FORMATOPTION_maskprefix | FORMATOPTION_masksuffix | FORMATOPTION_printcompressed | FORMATOPTION_printuncompressed | FORMATOPTION_printfulluncompressed | FORMATOPTION_forceprefix },
+	{ FORMAT_ipv6addr       , FORMATOPTION_machinereadable | FORMATOPTION_printlowercase | FORMATOPTION_printuppercase | FORMATOPTION_printprefix | FORMATOPTION_printsuffix | FORMATOPTION_maskprefix | FORMATOPTION_masksuffix | FORMATOPTION_printcompressed | FORMATOPTION_printuncompressed | FORMATOPTION_printfulluncompressed | FORMATOPTION_forceprefix | FORMATOPTION_print_iid_var },
 	{ FORMAT_mac            , FORMATOPTION_printlowercase | FORMATOPTION_printuppercase },
 	{ FORMAT_eui64          , FORMATOPTION_printlowercase | FORMATOPTION_printuppercase },
 	{ FORMAT_base85         , 0 },
@@ -231,6 +242,7 @@ typedef struct {
 #define ACTION_NUM_prefix_mac_to_ipv6	4
 #define ACTION_NUM_anonymize		5
 #define ACTION_NUM_6rd_local_prefix	6
+#define ACTION_NUM_filter		7
 #define ACTION_NUM_undefined		31
 
 #define ACTION_auto			(uint32_t) 0x0
@@ -240,6 +252,7 @@ typedef struct {
 #define ACTION_prefix_mac_to_ipv6	(uint32_t) (1 << ACTION_NUM_prefix_mac_to_ipv6)
 #define ACTION_anonymize		(uint32_t) (1 << ACTION_NUM_anonymize)
 #define ACTION_6rd_local_prefix		(uint32_t) (1 << ACTION_NUM_6rd_local_prefix)
+#define ACTION_filter			(uint32_t) (1 << ACTION_NUM_filter)
 #define ACTION_undefined		(uint32_t) (1 << ACTION_NUM_undefined)
 
 typedef struct {
@@ -250,13 +263,14 @@ typedef struct {
 } s_action;
 
 /*@unused@*/ static const s_action ipv6calc_actionstrings[] = {
-	{ ACTION_auto                 , "auto"           , "Automatic selection of action (default)", "" },
-	{ ACTION_mac_to_eui64         , "geneui64"       , "Converts a MAC address to an EUI-64 address", "" },
-	{ ACTION_ipv4_to_6to4addr     , "conv6to4"       , "Converts IPv4 address <-> 6to4 IPv6 address (prefix)", "" },
-	{ ACTION_iid_token_to_privacy , "genprivacyiid"  , "Generates a privacy interface ID out of a given one and a token", "" },
-	{ ACTION_prefix_mac_to_ipv6   , "prefixmac2ipv6" , "Generates an IPv6 address out of a prefix and a MAC address", "" },
-	{ ACTION_anonymize            , "anonymize"      , "Anonymize IPv4/IPv6 address without loosing much information", "" },
-	{ ACTION_6rd_local_prefix     , "6rd_local_prefix"  , "Calculate the 6rd prefix from given IPv6 prefix & relay prefix and IPv4", "" },
+	{ ACTION_auto                 , "auto"            , "Automatic selection of action (default)", "" },
+	{ ACTION_mac_to_eui64         , "geneui64"        , "Converts a MAC address to an EUI-64 address", "" },
+	{ ACTION_ipv4_to_6to4addr     , "conv6to4"        , "Converts IPv4 address <-> 6to4 IPv6 address (prefix)", "" },
+	{ ACTION_iid_token_to_privacy , "genprivacyiid"   , "Generates a privacy interface ID out of a given one (arg1) and a token (arg2)", "" },
+	{ ACTION_prefix_mac_to_ipv6   , "prefixmac2ipv6"  , "Generates an IPv6 address out of a prefix and a MAC address", "" },
+	{ ACTION_anonymize            , "anonymize"       , "Anonymize IPv4/IPv6 address without loosing much information", "" },
+	{ ACTION_6rd_local_prefix     , "6rd_local_prefix", "Calculate the 6rd prefix from given IPv6 prefix & relay prefix and IPv4", "" },
+	{ ACTION_filter	              , "filter"          , "Filter addresses related to filter options", "" },
 };
 
 /* Possible action option map */
@@ -271,4 +285,23 @@ typedef struct {
 	{ ACTION_6rd_local_prefix	, CMD_6rd_relay_prefix }
 };
 
+
+/* anonymization set */
+typedef struct {
+	char name[32];
+	char name_short[3];
+	int mask_ipv4;
+	int mask_ipv6;
+	int mask_iid;
+	int mask_mac;
+	int method;	// 1=anonymize, 2=zeroize
+} s_ipv6calc_anon_set;
+
+/* anonymization methods */
+typedef struct {
+	char name[32];
+	int method;
+	char description[128];
+} s_ipv6calc_anon_methods;
+
 #endif
