@@ -1,8 +1,8 @@
 /*
  * Project    : ipv6calc
  * File       : libeui64.h
- * Version    : $Id: libeui64.h,v 1.2 2002/03/24 16:58:21 peter Exp $
- * Copyright  : 2001-2002 by Peter Bieringer <pb (at) bieringer.de>
+ * Version    : $Id: libeui64.h,v 1.4 2013/04/10 18:30:52 ds6peter Exp $
+ * Copyright  : 2001-2013 by Peter Bieringer <pb (at) bieringer.de>
  *
  * Information:
  *  Header file for libeu64.c
@@ -24,3 +24,6 @@ typedef struct {
 
 /* prototypes */
 extern int create_eui64_from_mac(ipv6calc_ipv6addr *ipv6addrp, ipv6calc_macaddr *macaddrp);
+extern int libeui64_addr_to_eui64addrstruct(char *addrstring, char *resultstring, ipv6calc_eui64addr *eui64addrp);
+extern void libeui64_clear(ipv6calc_eui64addr *eui64addrp);
+extern void libeui64_clearall(ipv6calc_eui64addr *eui64addrp);
