@@ -1,8 +1,8 @@
 /*
  * Project    : ipv6calc/lib
  * File       : libipv4addr.c
- * Version    : $Id: libipv4addr.c,v 1.28 2012/02/05 09:03:34 peter Exp $
- * Copyright  : 2002-2012 by Peter Bieringer <pb (at) bieringer.de> except the parts taken from kernel source
+ * Version    : $Id: libipv4addr.c,v 1.38 2013/05/12 07:34:04 ds6peter Exp $
+ * Copyright  : 2002-2013 by Peter Bieringer <pb (at) bieringer.de> except the parts taken from kernel source
  *
  * Information:
  *  Function library for IPv4 address handling
@@ -236,12 +236,84 @@ void ipv4addr_clearall(ipv6calc_ipv4addr *ipv4addrp) {
 
 
 /*
- * function gets type of an IPv4 address
+ * function copies the IPv4 structure
  *
- *   to be implemented...
+ * in:  ipv64ddrp  = pointer to IPv4 address structure
+ * mod: ipv64ddrp2 = pointer to IPv4 address structure
+ */
+#define DEBUG_function_name "libipv4addr/ipv4addr_copy"
+void ipv4addr_copy(ipv6calc_ipv4addr *ipv4addrp_dst, const ipv6calc_ipv4addr *ipv4addrp_src) {
+
+	*(ipv4addrp_dst) = *(ipv4addrp_src);
+	
+	return;
+};
+#undef DEBUG_function_name
+
+
+/*
+ * function gets type of an IPv4 address
  */
 uint32_t ipv4addr_gettype(/*@unused@*/ const ipv6calc_ipv4addr *ipv4addrp) {
-	return (0);
+	uint32_t type = 0;
+	uint32_t ipv4 = ipv4addr_getdword(ipv4addrp);
+
+	if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+		fprintf(stderr, "%s/%s: got IPv4 address: 0x%08x\n", __FILE__, __func__, ipv4);
+	};
+
+	if (ipv4 == 0x00000000u) {
+		type = IPV4_ADDR_ANY;
+	} else if ((ipv4 & 0xff000000u) == 0x0a000000u) {
+		// 10.0.0.0/8 (RFC 1918)
+		type = IPV4_ADDR_UNICAST | IPV4_ADDR_SITELOCAL;
+	} else if ((ipv4 & 0xff000000u) == 0x7f000000u) {
+		// 127.0.0.0/8 (RFC 1122)
+		type = IPV4_ADDR_LOOPBACK;
+	} else if ((ipv4 & 0xffff0000u) == 0xa9fe0000u) {
+		// 169.254.0.0/16 (RFC 1918)
+		type = IPV4_ADDR_UNICAST | IPV4_ADDR_ZEROCONF | IPV4_ADDR_SITELOCAL;
+	} else if ((ipv4 & 0xfff00000u) == 0xac100000u) {
+		// 172.16.0.0/12 (RFC 1918)
+		type = IPV4_ADDR_UNICAST | IPV4_ADDR_SITELOCAL;
+	} else if ((ipv4 & 0xffff0000u) == 0xc0a80000u) {
+		// 192.168.0.0/16 (RFC 1918)
+		type = IPV4_ADDR_UNICAST | IPV4_ADDR_SITELOCAL;
+	} else if ((ipv4 & 0xffffff00u) == 0xc0000000u) {
+		// 192.0.0.0/24 (RFC 5736)
+		type = IPV4_ADDR_RESERVED;
+	} else if ((ipv4 & 0xffffff00u) == 0xc0000200u) {
+		// 192.0.2.0/24 (RFC 3330)
+		type = IPV4_ADDR_RESERVED;
+	} else if ((ipv4 & 0xffffff00u) == 0xc0586300u) {
+		// 192.88.99.0/24 (RFC 3068)
+		type = IPV4_ADDR_ANYCAST | IPV4_ADDR_6TO4RELAY;
+	} else if ((ipv4 & 0xfffe0000u) == 0xc6120000u) {
+		// 198.18.0.0/15 (RFC 2544)
+		type = IPV4_ADDR_RESERVED;
+	} else if ((ipv4 & 0xffffff00u) == 0xc6336400u) {
+		// 198.51.100.0/24 (RFC 5737)
+		type = IPV4_ADDR_RESERVED;
+	} else if ((ipv4 & 0xffffff00u) == 0xcb007100u) {
+		// 203.0.113.0/24 (RFC 5737)
+		type = IPV4_ADDR_RESERVED;
+	} else if ((ipv4 & 0xf0000000u) == 0xe0000000u) {
+		// 224.0.0.0/4 (RFC 3171)
+		type = IPV4_ADDR_MULTICAST;
+	} else if ((ipv4 & 0xffffffffu) == 0xffffffffu) {
+		// 255.255.255.255/32
+		type = IPV4_ADDR_BROADCAST;
+	} else if ((ipv4 & 0xf0000000u) == 0xf0000000u) {
+		// 240.0.0.0/4 (RFC 1112)
+		type = IPV4_ADDR_RESERVED;
+	} else {
+		type = IPV4_ADDR_UNICAST;
+	};
+
+	if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+		fprintf(stderr, "%s/%s: return typeinfo: 0x%08x\n", __FILE__, __func__, type);
+	};
+	return(type);
 };
 
 
@@ -282,7 +354,7 @@ int addr_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 		return (1);
 	};
 
-	strncpy(tempstring, addrstring, sizeof(tempstring) - 1);
+	snprintf(tempstring, sizeof(tempstring) - 1, "%s", addrstring);
 	
 	ipv4addr_clearall(ipv4addrp);
 
@@ -377,7 +449,7 @@ int addr_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 	typeinfo = ipv4addr_gettype(ipv4addrp); 
 
 	if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
-		fprintf(stderr, "%s: Got scope %02x\n", DEBUG_function_name, (unsigned int) typeinfo);
+		fprintf(stderr, "%s: Got typeinfo: 0x%08x\n", DEBUG_function_name, typeinfo);
 	};
 
 	ipv4addrp->scope = typeinfo;
@@ -414,10 +486,10 @@ int addrhex_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6ca
 		fprintf(stderr, "%s: Got input '%s'\n", DEBUG_function_name,  addrstring);
 	};
 
-	if ((strlen(addrstring) < 8)  || (strlen(addrstring) > 11)) {
+	if ((strlen(addrstring) < 7)  || (strlen(addrstring) > 11)) {
 		/* min: ffffffff */
 		/* max: ffffffff/32 */
-		snprintf(resultstring, NI_MAXHOST - 1, "Error in given hex notated IPv4 address, has not 8 to 11 chars!");
+		snprintf(resultstring, NI_MAXHOST - 1, "Error in given hex notated IPv4 address, has not 7 to 11 chars!");
 		return (1);
 	};
 
@@ -426,7 +498,7 @@ int addrhex_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6ca
 		return (1);
 	};
 
-	strncpy(tempstring, addrstring, sizeof(tempstring) - 1);
+	snprintf(tempstring, sizeof(tempstring) - 1, "%s", addrstring);
 	
 	ipv4addr_clearall(ipv4addrp);
 
@@ -450,7 +522,7 @@ int addrhex_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6ca
 	};
 
 	/* check length 3 */
-	if (strlen(addronlystring) != 8) {
+	if ((flag_reverse == 0 && (strlen(addronlystring) < 7 || strlen(addronlystring) > 8)) || (flag_reverse != 0 && strlen(addronlystring) != 8)) {
 		snprintf(resultstring, NI_MAXHOST - 1, "Error, given hexadecimal IPv4 address '%s' is not valid (not proper length)!", addronlystring);
 		retval = 1;
 		return (retval);
@@ -465,7 +537,11 @@ int addrhex_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6ca
 	if (flag_reverse != 0)  {
 		result = sscanf(addronlystring, "%2x%2x%2x%2x", &compat[3], &compat[2], &compat[1], &compat[0]);
 	} else {
-		result = sscanf(addronlystring, "%2x%2x%2x%2x", &compat[0], &compat[1], &compat[2], &compat[3]);
+		if (strlen(addronlystring) == 7) {
+			result = sscanf(addronlystring, "%1x%2x%2x%2x", &compat[0], &compat[1], &compat[2], &compat[3]);
+		} else {
+			result = sscanf(addronlystring, "%2x%2x%2x%2x", &compat[0], &compat[1], &compat[2], &compat[3]);
+		};
 	};
 	
 	for ( i = 0; i <= 3; i++ ) {
@@ -502,7 +578,7 @@ int addrhex_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6ca
 	typeinfo = ipv4addr_gettype(ipv4addrp); 
 
 	if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
-		fprintf(stderr, "%s: Got scope %02x\n", DEBUG_function_name, (unsigned int) typeinfo);
+		fprintf(stderr, "%s: Got typeinfo: 0x%08x\n", DEBUG_function_name, typeinfo);
 	};
 
 	ipv4addrp->scope = typeinfo;
@@ -841,7 +917,7 @@ int libipv4addr_to_octal(const ipv6calc_ipv4addr *ipv4addrp, char *resultstring,
  * ret: ==0: ok, !=0: error
  */
 #define DEBUG_function_name "libipv4addr/ipv4addrstruct_to_hex"
-int libipv4addr_to_hex(const ipv6calc_ipv4addr *ipv4addrp, char *resultstring, const uint32_t formatoptions) {
+int libipv4addr_to_hex(const ipv6calc_ipv4addr *ipv4addrp, char *resultstring, /*@unused@*/ const uint32_t formatoptions) {
 	int retval = 1;
 	char tempstring[NI_MAXHOST];
 
@@ -873,6 +949,7 @@ void libipv4addr_anonymize(ipv6calc_ipv4addr *ipv4addrp, unsigned int mask) {
 	if (mask == 0) {
 		/* clear IPv4 address: 0.0.0.0 */
 		ipv4addr_clear(ipv4addrp);
+		ipv4addrp->flag_valid = 1;
 	} else if (mask == 32) {
 		/* nothing to do */
 	} else if (mask < 1 || mask > 31) {
@@ -897,3 +974,144 @@ void libipv4addr_anonymize(ipv6calc_ipv4addr *ipv4addrp, unsigned int mask) {
 };
 #undef DEBUG_function_name
 
+
+/*
+ * clear filter IPv4 address
+ *
+ * in : *filter    = filter structure
+ */
+void ipv4addr_filter_clear(s_ipv6calc_filter_ipv4addr *filter) {
+	filter->active = 0;
+	filter->typeinfo_must_have = 0;
+	filter->typeinfo_may_not_have = 0;
+	return;
+};
+
+
+/*
+ * parse filter IPv4 address
+ *
+ * in : *filter    = filter structure
+ * ret: 0:found 1:skip 2:problem
+ */
+int ipv4addr_filter_parse(s_ipv6calc_filter_ipv4addr *filter, const char *token) {
+	int i, result = 1, negate = 0, offset = 0;
+	const char *prefix = "ipv4";
+	const char *prefixdot = "ipv4.";
+
+	if (token == NULL) {
+		return (result);
+	};
+
+	if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+		fprintf(stderr, "%s/%s: input: %s\n", __FILE__, __func__, token);
+	};
+
+	if (token[0] == '^') {
+		if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+			fprintf(stderr, "%s/%s: found negate prefix in token: %s\n", __FILE__, __func__, token);
+		};
+
+		negate = 1;
+		offset += 1;
+	};
+
+	if (strcmp(token + offset, prefix) == 0) {
+		/* any */
+		if (negate == 1) {
+			filter->typeinfo_may_not_have = ~IPV4_ADDR_ANY;
+		} else {
+			filter->typeinfo_must_have = IPV4_ADDR_ANY;
+		};
+		filter->active = 1;
+		result = 0;
+		goto END_ipv4addr_filter_parse;
+
+	} else if (strncmp(token + offset, prefixdot, strlen(prefixdot)) == 0) {
+		/* prefix with dot found */
+		offset += strlen(prefixdot);
+		result = 2; /* token with prefix, result into problem if not found */
+
+	} else if (strstr(token, ".") != NULL) {
+		/* other prefix */
+		if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+			fprintf(stderr, "%s/%s: prefix did not match: %s\n", __FILE__, __func__, token + offset);
+		};
+		return(1);
+	};
+
+	for (i = 0; i < (int) (sizeof(ipv6calc_ipv4addrtypestrings) / sizeof(ipv6calc_ipv4addrtypestrings[0])); i++ ) {
+		if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+			fprintf(stderr, "%s/%s: check token against: %s\n", __FILE__, __func__, ipv6calc_ipv4addrtypestrings[i].token);
+		};
+
+		if (strcmp(ipv6calc_ipv4addrtypestrings[i].token, token + offset) == 0) {
+			if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+				fprintf(stderr, "%s/%s: token match: %s\n", __FILE__, __func__, ipv6calc_ipv4addrtypestrings[i].token);
+			};
+
+			if (negate == 1) {
+				filter->typeinfo_may_not_have |= ipv6calc_ipv4addrtypestrings[i].number;
+			} else {
+				filter->typeinfo_must_have |= ipv6calc_ipv4addrtypestrings[i].number;
+			};
+			filter->active = 1;
+			result = 0;
+			break;
+		};
+	};
+
+	if (result != 0) {
+		if ((ipv6calc_debug & DEBUG_libipv4addr) != 0) {
+			fprintf(stderr, "%s/%s: token not supported: %s\n", __FILE__, __func__, token);
+		};
+		return (result);
+	};
+
+END_ipv4addr_filter_parse:
+	if ((ipv6calc_debug & DEBUG_libipv4addr) != 0) {
+		fprintf(stderr, "%s/%s: filter 'must_have'   : 0x%08x\n", __FILE__, __func__, filter->typeinfo_must_have);
+		fprintf(stderr, "%s/%s: filter 'may_not_have': 0x%08x\n", __FILE__, __func__, filter->typeinfo_may_not_have);
+		fprintf(stderr, "%s/%s: filter 'active': %d\n", __FILE__, __func__, filter->active);
+	};
+
+	return (result);
+};
+
+
+/*
+ * filter IPv4 address
+ *
+ * in : *ipv4addrp = IPv4 address structure
+ * in : *filter    = filter structure
+ * ret: 0=match 1=not match
+ */
+int ipv4addr_filter(const ipv6calc_ipv4addr *ipv4addrp, const s_ipv6calc_filter_ipv4addr *filter) {
+	uint32_t typeinfo;
+
+	if (filter->active == 0) {
+		if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+			fprintf(stderr, "%s/%s: no filter active (SKIP)\n", __FILE__, __func__);
+		};
+		return (1);
+	};
+
+	if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+		fprintf(stderr, "%s/%s: start\n", __FILE__, __func__);
+	};
+
+	/* get type */
+	typeinfo = ipv4addr_gettype(ipv4addrp);
+
+	if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+		fprintf(stderr, "%s/%s: compare typeinfo against must_have: 0x%08x/0x%08x\n", __FILE__, __func__, typeinfo, filter->typeinfo_must_have);
+	};
+
+	if ((typeinfo & filter->typeinfo_must_have) == filter->typeinfo_must_have) {
+		if ((typeinfo & filter->typeinfo_may_not_have) == 0) {
+			return (0);
+		};
+	};
+
+	return (1);
+};
