@@ -1,15 +1,15 @@
 /*
  * Project    : ipv6calc/lib
  * File       : libipv4addr.h
- * Version    : $Id: libipv4addr.h,v 1.18 2011/09/16 18:05:13 peter Exp $
- * Copyright  : 2002-2007 by Peter Bieringer <pb (at) bieringer.de> except the parts taken from kernel source
+ * Version    : $Id: libipv4addr.h,v 1.24 2013/03/25 21:29:47 ds6peter Exp $
+ * Copyright  : 2002-2012 by Peter Bieringer <pb (at) bieringer.de> except the parts taken from kernel source
  * License    : GNU GPL v2
  *
  * Information:
  *  Header file for libipv4addr.c
  */ 
 
-#include "ipv6calc_inttypes.h"
+#include "ipv6calctypes.h"
 #include <netinet/in.h>
 
 /* typedefs */
@@ -42,6 +42,52 @@ typedef struct {
 	const unsigned int end;
 } s_ipv4addr_assignment_hint;
 
+
+typedef struct {
+	int active;	
+	uint32_t typeinfo_must_have;
+	uint32_t typeinfo_may_not_have;
+	/* others coming next */
+} s_ipv6calc_filter_ipv4addr;
+
+
+
+/*
+ * IPv4 address type definitions 
+ */
+
+#define IPV4_ADDR_ANY				(uint32_t) 0x00000000U
+
+#define IPV4_ADDR_UNICAST			(uint32_t) 0x00000001U	
+#define IPV4_ADDR_MULTICAST			(uint32_t) 0x00000002U	
+#define IPV4_ADDR_ANYCAST			(uint32_t) 0x00000004U	
+#define IPV4_ADDR_BROADCAST			(uint32_t) 0x00000008U	
+#define IPV4_ADDR_LOOPBACK			(uint32_t) 0x00000010U
+#define IPV4_ADDR_UNKNOWN			(uint32_t) 0x00000400U
+#define IPV4_ADDR_RESERVED			(uint32_t) 0x00000800U
+
+#define IPV4_ADDR_ZEROCONF			(uint32_t) 0x00001000U
+#define IPV4_ADDR_SITELOCAL			(uint32_t) 0x00002000U
+
+#define IPV4_ADDR_6TO4RELAY			(uint32_t) 0x01000000U
+
+
+/* text representation */
+/*@unused@*/ static const s_type ipv6calc_ipv4addrtypestrings[] = {
+	{ IPV4_ADDR_ANY			, "any" },
+	{ IPV4_ADDR_UNICAST		, "unicast" },
+	{ IPV4_ADDR_MULTICAST		, "multicast" },
+	{ IPV4_ADDR_ANYCAST		, "anycast" },
+	{ IPV4_ADDR_BROADCAST		, "broadcast" },
+	{ IPV4_ADDR_LOOPBACK		, "loopback" },
+	{ IPV4_ADDR_UNKNOWN		, "unknown" },
+	{ IPV4_ADDR_RESERVED		, "reserved" },
+	{ IPV4_ADDR_ZEROCONF		, "zeroconf" },
+	{ IPV4_ADDR_SITELOCAL		, "site-local" },
+	{ IPV4_ADDR_6TO4RELAY		, "6to4relay" }
+};
+
+
 /* Registries */
 #define IPV4_ADDR_REGISTRY_IANA		0x02
 #define IPV4_ADDR_REGISTRY_APNIC	0x03
@@ -52,6 +98,9 @@ typedef struct {
 #define IPV4_ADDR_REGISTRY_RESERVED	0x0e
 #define IPV4_ADDR_REGISTRY_UNKNOWN	0x0f
 
+
+
+
 #endif
 
 
@@ -66,6 +115,7 @@ extern void ipv4addr_setdword(ipv6calc_ipv4addr *ipv4addrp, const unsigned int v
 
 extern void ipv4addr_clear(ipv6calc_ipv4addr *ipv4addrp);
 extern void ipv4addr_clearall(ipv6calc_ipv4addr *ipv4addrp);
+extern void ipv4addr_copy(ipv6calc_ipv4addr *ipv4addrp_dst, const ipv6calc_ipv4addr *ipv4addrp_src);
 
 extern uint32_t ipv4addr_gettype(const ipv6calc_ipv4addr *ipv4addrp);
 
@@ -83,4 +133,8 @@ extern int libipv4addr_to_reversestring(ipv6calc_ipv4addr *ipv4addrp, char *resu
 extern int libipv4addr_to_octal(const ipv6calc_ipv4addr *ipv4addrp, char *resultstring, const uint32_t formatoptions);
 extern int libipv4addr_to_hex(const ipv6calc_ipv4addr *ipv4addrp, char *resultstring, const uint32_t formatoptions);
 
-extern void libipv4addr_anonymize(ipv6calc_ipv4addr *ipv4addrp, unsigned int mask);
+extern void libipv4addr_anonymize(ipv6calc_ipv4addr *ipv4addrp, const unsigned int mask);
+
+extern int ipv4addr_filter(const ipv6calc_ipv4addr *ipv4addrp, const s_ipv6calc_filter_ipv4addr *filter);
+extern int ipv4addr_filter_parse(s_ipv6calc_filter_ipv4addr *filter, const char *token);
+extern void ipv4addr_filter_clear(s_ipv6calc_filter_ipv4addr *filter);
