@@ -1,8 +1,8 @@
 /*
  * Project    : ipv6calc/lib
  * File       : libipv6calc.c
- * Version    : $Id: libipv6calc.c,v 1.23 2012/01/10 20:50:16 peter Exp $
- * Copyright  : 2001-2011 by Peter Bieringer <pb (at) bieringer.de>
+ * Version    : $Id: libipv6calc.c,v 1.31 2013/05/12 07:23:12 ds6peter Exp $
+ * Copyright  : 2001-2013 by Peter Bieringer <pb (at) bieringer.de>
  *
  * Information:
  *  Function library for some tools
@@ -14,6 +14,7 @@
 #include <ctype.h>
 
 #include "ipv6calctypes.h"
+#include "libipv6calc.h"
 #include "libipv6calcdebug.h"
 #include "librfc1924.h"
 #include "librfc2874.h"
@@ -141,8 +142,8 @@ void string_to_reverse_dotted(char *string) {
 #define DEBUG_function_name "libipv6calc/autodetectinput"
 uint32_t libipv6calc_autodetectinput(const char *string) {
 	uint32_t type = FORMAT_auto_noresult;
-	int i, j = 0, result;
-	int numdots = 0, numcolons = 0, numdigits = 0, numxdigits = 0, numdashes = 0, numspaces = 0, numslashes = 0, numalnums = 0, numchar_s = 0, numpercents = 0;
+	int xdl, i, j = 0, result;
+	int numdots = 0, numcolons = 0, numdigits = 0, numxdigits = 0, numdashes = 0, numspaces = 0, numslashes = 0, numalnums = 0, numchar_s = 0, numpercents = 0, numcolonsdouble = 0, xdigitlen_max = 0, xdigitlen_min = 0;
 	char resultstring[NI_MAXHOST];
 	size_t length;
 
@@ -153,32 +154,62 @@ uint32_t libipv6calc_autodetectinput(const char *string) {
 		goto END_libipv6calc_autodetectinput;
 	};
 
+	xdl = 0;
 	for (i = 0; i < (int) length; i++) {
 		if (string[i] == '.') { numdots++; };
-		if (string[i] == ':') { numcolons++; };
+		if (string[i] == ':') {
+			numcolons++;
+			if (i < (int) length + 1) {
+				/* check for double colons */
+				if (string[i+1] == ':') {
+					numcolonsdouble++;
+					numcolons++;
+					i++;
+				};
+			};
+		};
 		if (string[i] == '-') { numdashes++; };
 		if (string[i] == '/') { numslashes++; };
 		if (string[i] == ' ') { numspaces++; };
 		if (string[i] == '%') { numpercents++; };
 		if (string[i] == 's') { numchar_s++; };
 		if (isdigit((int) string[i])) { numdigits++; };
-		if (isxdigit((int) string[i])) { numxdigits++; };
+		if (isxdigit((int) string[i])) {
+			numxdigits++;
+			xdl++;
+		} else {
+			if (xdigitlen_max == 0 && xdigitlen_min == 0) {
+				// init
+				xdigitlen_max = xdl;
+				xdigitlen_min = xdl;
+			};
+			if (xdl > xdigitlen_max) {
+				xdigitlen_max = xdl;
+			};
+			if (xdl < xdigitlen_min) {
+				xdigitlen_min = xdl;
+			};
+			xdl = 0;
+		};
 		if (isalnum((int) string[i])) { numalnums++; };
 	};
 
 	if ( (ipv6calc_debug & DEBUG_libipv6calc) != 0 ) {
 		fprintf(stderr, "%s: Autodetection source:\n", DEBUG_function_name);
-		fprintf(stderr, "%s:  numdots    :%d\n", DEBUG_function_name, numdots);
-		fprintf(stderr, "%s:  numcolons  :%d\n", DEBUG_function_name, numcolons);
-		fprintf(stderr, "%s:  numdashes  :%d\n", DEBUG_function_name, numdashes);
-		fprintf(stderr, "%s:  numspaces  :%d\n", DEBUG_function_name, numspaces);
-		fprintf(stderr, "%s:  numslashes :%d\n", DEBUG_function_name, numslashes);
-		fprintf(stderr, "%s:  numdigits  :%d\n", DEBUG_function_name, numdigits);
-		fprintf(stderr, "%s:  numxdigits :%d\n", DEBUG_function_name, numxdigits);
-		fprintf(stderr, "%s:  numalnums  :%d\n", DEBUG_function_name, numalnums);
-		fprintf(stderr, "%s:  numpercents:%d\n", DEBUG_function_name, numpercents);
-		fprintf(stderr, "%s:  numchar_s  :%d\n", DEBUG_function_name, numchar_s);
-		fprintf(stderr, "%s:  length     :%d\n", DEBUG_function_name, (int) length);
+		fprintf(stderr, "%s:  numdots        :%d\n", DEBUG_function_name, numdots);
+		fprintf(stderr, "%s:  numcolons      :%d\n", DEBUG_function_name, numcolons);
+		fprintf(stderr, "%s:  numcolonsdouble:%d\n", DEBUG_function_name, numcolonsdouble);
+		fprintf(stderr, "%s:  numdashes      :%d\n", DEBUG_function_name, numdashes);
+		fprintf(stderr, "%s:  numspaces      :%d\n", DEBUG_function_name, numspaces);
+		fprintf(stderr, "%s:  numslashes     :%d\n", DEBUG_function_name, numslashes);
+		fprintf(stderr, "%s:  numdigits      :%d\n", DEBUG_function_name, numdigits);
+		fprintf(stderr, "%s:  numxdigits     :%d\n", DEBUG_function_name, numxdigits);
+		fprintf(stderr, "%s:  numalnums      :%d\n", DEBUG_function_name, numalnums);
+		fprintf(stderr, "%s:  numpercents    :%d\n", DEBUG_function_name, numpercents);
+		fprintf(stderr, "%s:  numchar_s      :%d\n", DEBUG_function_name, numchar_s);
+		fprintf(stderr, "%s:  xdigit len max :%d\n", DEBUG_function_name, xdigitlen_max);
+		fprintf(stderr, "%s:  xdigit len min :%d\n", DEBUG_function_name, xdigitlen_min);
+		fprintf(stderr, "%s:  length         :%d\n", DEBUG_function_name, (int) length);
 	};
 
 	if ( length == 20 && numdots == 0 && numcolons == 0 ) {
@@ -223,14 +254,30 @@ uint32_t libipv6calc_autodetectinput(const char *string) {
 		goto END_libipv6calc_autodetectinput;
 	};
 	
-	if (length == 8 && numxdigits == 8 && numdots == 0 && numcolons == 0) {
-		/* IPv4 hexadecimal: xxxxxxxx */
+	if (((length == 8 && numxdigits == 8) || (length == 7 && numxdigits == 7)) && numdots == 0 && numcolons == 0) {
+		/* IPv4 hexadecimal: xxxxxxxx or xxxxxxx */
 		type = FORMAT_ipv4hex;
 		goto END_libipv6calc_autodetectinput;
 	};
 	
-	if ((length >= 11 && length <= 17 && numxdigits >= 6 && numxdigits <= 12 && numdots == 0 && ( (numcolons == 5 && numdashes == 0 && numspaces == 0) || (numcolons == 0 && numdashes == 5 && numspaces == 0) || (numcolons == 0 && numdashes == 0 && numspaces == 5))) || (length == 13 && numcolons == 0 && numdashes == 1 && numspaces == 0 && numxdigits == 12) || (length == 12 && numcolons == 0 && numdashes == 0 && numspaces == 0 && numxdigits == 12)) {
-		/* MAC 00:00:00:00:00:00 or 00-00-00-00-00-00 or "xx xx xx xx xx xx" or "xxxxxx-xxxxxx" or xxxxxxxxxxxx */
+	if ((length >= 11 && length <= 17 && numxdigits >= 6 && numxdigits <= 12 && numdots == 0 && ( (numcolons == 5 && numdashes == 0 && numspaces == 0)
+	   	 || (numcolons == 0 && numdashes == 5 && numspaces == 0)
+		    || (numcolons == 0 && numdashes == 0 && numspaces == 5))
+	    )
+	    || (length == 13 && numcolons == 0 && numdashes == 1 && numspaces == 0 && numxdigits == 12)
+	    || (length == 12 && numcolons == 0 && numdashes == 0 && numspaces == 0 && numxdigits == 12)
+	    || (length == 14 && numdots == 2 && numxdigits ==12 && xdigitlen_min == 4 && xdigitlen_max == 4)
+	   ) {
+		/* MAC 00:00:00:00:00:00 or 00-00-00-00-00-00 or "xx xx xx xx xx xx" or "xxxxxx-xxxxxx" or xxxxxxxxxxxx or xxxx.xxxx.xxxx */
+
+	    	if (length == 14 && numdots == 2 && numxdigits ==12 && xdigitlen_min == 4 && xdigitlen_max == 4) {
+			// xxxx.xxxx.xxxx
+			type = FORMAT_mac;
+			if ( (ipv6calc_debug & DEBUG_libipv6calc) != 0 ) {
+				fprintf(stderr, "%s: Autodetection found type: mac\n", DEBUG_function_name);
+			};
+			goto END_libipv6calc_autodetectinput;
+		};
 
 		if ( (ipv6calc_debug & DEBUG_libipv6calc) != 0 ) {
 			fprintf(stderr, "%s:  check FORMAT_mac\n", DEBUG_function_name);
@@ -239,58 +286,58 @@ uint32_t libipv6calc_autodetectinput(const char *string) {
 		/* Check whether minimum 1 xdigit is between colons, dashes, spaces */
 		if (numcolons == 0 && numdashes == 1 && numspaces == 0 && numxdigits == 12) {
 
-		/* Check xxxxxx-xxxxxx */
-		j = 0;
-		for (i = 0; i < (int) length; i++) {
-			if (isxdigit((int) string[i])) {
-				j++;
-				if ( j > 6 ) {
-					/* more than 6 xdigits */
-					j = -1;
-					break;
+			/* Check xxxxxx-xxxxxx */
+			j = 0;
+			for (i = 0; i < (int) length; i++) {
+				if (isxdigit((int) string[i])) {
+					j++;
+					if ( j > 6 ) {
+						/* more than 6 xdigits */
+						j = -1;
+						break;
+					};
+					continue;
+				} else if (string[i] == '-' ) {
+					if ( j == 0 ) {
+						/* dash follow dash */
+						j = -1;
+						break;
+					};
+					j = 0;
+					continue;
 				};
-				continue;
-		       	} else if (string[i] == '-' ) {
-				if ( j == 0 ) {
-					/* dash follow dash */
-					j = -1;
-					break;
-				};
-				j = 0;
-				continue;
-		       	};
-			/* normally not reached */
-			j = -1;
-			break;
-		};
+				/* normally not reached */
+				j = -1;
+				break;
+			};
 
 		} else if (numcolons == 0 && numdashes == 0 && numspaces == 0 && numxdigits == 12) {
 			/* nothing more to check */
 		} else {
 
-		j = 0;
-		for (i = 0; i < (int) length; i++) {
-			if (isxdigit((int) string[i])) {
-				j++;
-				if ( j > 2 ) {
-					/* more than 2 xdigits */
-					j = -1;
-					break;
+			j = 0;
+			for (i = 0; i < (int) length; i++) {
+				if (isxdigit((int) string[i])) {
+					j++;
+					if ( j > 2 ) {
+						/* more than 2 xdigits */
+						j = -1;
+						break;
+					};
+					continue;
+				} else if (string[i] == ':' || string[i] == '-' || string[i] == ' ') {
+					if ( j == 0 ) {
+						/* colon/dash/space follows colon/dash/space */
+						j = -1;
+						break;
+					};
+					j = 0;
+					continue;
 				};
-				continue;
-		       	} else if (string[i] == ':' || string[i] == '-' || string[i] == ' ') {
-				if ( j == 0 ) {
-					/* colon/dash/space follows colon/dash/space */
-					j = -1;
-					break;
-				};
-				j = 0;
-				continue;
-		       	};
-			/* normally not reached */
-			j = -1;
-			break;
-		};
+				/* normally not reached */
+				j = -1;
+				break;
+			};
 
 		}; /* end of if */
 
@@ -302,6 +349,52 @@ uint32_t libipv6calc_autodetectinput(const char *string) {
 			goto END_libipv6calc_autodetectinput;
 		};
 	};
+
+	if ((length >= 15 && length <= 23 && numxdigits >= 8 && numxdigits <= 16 && numdots == 0 && ( (numcolons == 7 && numdashes == 0 && numspaces == 0) || (numcolons == 0 && numdashes == 7 && numspaces == 0) || (numcolons == 0 && numdashes == 0 && numspaces == 7))) || (length == 16 && numcolons == 0 && numdashes == 0 && numspaces == 0 && numxdigits == 16)) {
+		/* EUI-64 00:00:00:00:00:00:00:00 or 00-00-00-00-00-00-00-00 or "xx xx xx xx xx xx xx xx" or xxxxxxxxxxxxxxxx */
+
+		if ( (ipv6calc_debug & DEBUG_libipv6calc) != 0 ) {
+			fprintf(stderr, "%s:  check FORMAT_eui64\n", DEBUG_function_name);
+		};
+
+		if (numcolons == 0 && numdashes == 0 && numspaces == 0 && numxdigits == 16) {
+			/* nothing more to check */
+		} else {
+
+			j = 0;
+			for (i = 0; i < (int) length; i++) {
+				if (isxdigit((int) string[i])) {
+					j++;
+					if ( j > 2 ) {
+						/* more than 2 xdigits */
+						j = -1;
+						break;
+					};
+					continue;
+				} else if (string[i] == ':' || string[i] == '-' || string[i] == ' ') {
+					if ( j == 0 ) {
+						/* colon/dash/space follows colon/dash/space */
+						j = -1;
+						break;
+					};
+					j = 0;
+					continue;
+				};
+				/* normally not reached */
+				j = -1;
+				break;
+			};
+
+		}; /* end of if */
+
+		if ( j != -1 ) {
+			type = FORMAT_eui64;
+			if ( (ipv6calc_debug & DEBUG_libipv6calc) != 0 ) {
+				fprintf(stderr, "%s: Autodetection found type: eui64\n", DEBUG_function_name);
+			};
+			goto END_libipv6calc_autodetectinput;
+		};
+	};
 	
 	if (numcolons == 0 && numdots > 0 && numslashes == 0 && numspaces == 0 && (numalnums + numdots) == length) {
 		/* check for reverse nibble string */
@@ -318,6 +411,12 @@ uint32_t libipv6calc_autodetectinput(const char *string) {
 		};
 	};
 	
+	if ((numcolons == 3) && (numcolonsdouble == 0) && numdots == 0 && numslashes == 0 && numpercents == 0 && ((numcolons + numxdigits) == length)) {
+		/* IID */
+		type = FORMAT_iid;
+		goto END_libipv6calc_autodetectinput;
+	};
+
 	if (numcolons != 0 && numdots <= 3 && numslashes <= 1 && ((numpercents == 0 && (numdots + numcolons + numxdigits + numslashes) == length) || (numpercents == 1 && (numdots + numcolons + numxdigits + numslashes + numpercents) <= length))) {
 		/* hopefully an IPv6 address */
 		/* fe80::1 */
@@ -344,3 +443,128 @@ END_libipv6calc_autodetectinput:
 	return (type);
 };
 #undef DEBUG_function_name
+
+
+/*
+ * clear filter master address
+ *
+ * in : *filter    = filter structure
+ */
+void libipv6calc_filter_clear(s_ipv6calc_filter_master *filter_master) {
+	ipv4addr_filter_clear(&filter_master->filter_ipv4addr);
+	ipv6addr_filter_clear(&filter_master->filter_ipv6addr);
+        macaddr_filter_clear(&filter_master->filter_macaddr);
+	return;
+};
+
+
+/*
+ * function parses ipv6calc filter expression
+ *
+ * in : pointer to a string
+ * mod: master filter structure
+ * ret: success
+ */
+int libipv6calc_filter_parse(const char *expression, s_ipv6calc_filter_master *filter_master) {
+	char tempstring[NI_MAXHOST] = "";
+	char *charptr, *cptr, **ptrptr;
+	ptrptr = &cptr;
+	int r, token_used, result = 0;
+
+	snprintf(tempstring, NI_MAXHOST - 1, "%s", expression);
+
+	/* split expression */
+	charptr = strtok_r(tempstring, ",", ptrptr);
+	while (charptr != NULL) {
+		token_used = 0;
+
+		r = ipv4addr_filter_parse(&filter_master->filter_ipv4addr, charptr);
+		if (r == 0) {
+			token_used = 1;
+		} else if (r == 2) {
+			result = 1;
+			fprintf(stderr, "Unrecognized filter token: %s\n", charptr);
+		};
+
+		r = ipv6addr_filter_parse(&filter_master->filter_ipv6addr, charptr);
+		if (r == 0) {
+			token_used = 1;
+		} else if (r == 2) {
+			result = 1;
+			fprintf(stderr, "Unrecognized filter token: %s\n", charptr);
+		};
+
+		// r += macaddr_filter_parse(&filter_macaddr, charptr);
+
+		/* overall check */
+		if (token_used == 0) {
+			result = 1;
+			fprintf(stderr, "Unrecognized filter token: %s\n", charptr);
+		};
+
+		charptr = strtok_r(NULL, ",", ptrptr);
+	};
+
+	return (result);
+};
+
+
+/*
+ * return proper anonymization set by name
+ *
+ * in : *name = name of anonymization set
+ *      *ipv6calc_anon_set = pointer to anonymization set to be filled
+ * return: 1=not found, 0=success
+ */
+int libipv6calc_anon_set_by_name(s_ipv6calc_anon_set *ipv6calc_anon_set, const char *name) {
+	int i;
+
+	if (strlen(name) == 0) {
+		fprintf(stderr, "Name of anonyimization set is empty\n");
+		return 1;
+	};
+
+	if ( (ipv6calc_debug) != 0 ) {
+		fprintf(stderr, "%s/%s: search for anonymization set with name: %s\n", __FILE__, __func__, name);
+	};
+
+	for (i = 0; i < sizeof(ipv6calc_anon_set_list) / sizeof(s_ipv6calc_anon_set); i++) {
+		if ( (ipv6calc_debug) != 0 ) {
+			fprintf(stderr, "%s/%s: compare name: %s ? %s\n", __FILE__, __func__, name, ipv6calc_anon_set_list[i].name);
+		};
+
+		if ((strcmp(name, ipv6calc_anon_set_list[i].name) == 0) || (strcmp(name, ipv6calc_anon_set_list[i].name_short) == 0)) {
+			if ( (ipv6calc_debug) != 0 ) {
+				fprintf(stderr, "%s/%s: hit name: %s = %s\n", __FILE__, __func__, name, ipv6calc_anon_set_list[i].name);
+			};
+
+			memcpy(ipv6calc_anon_set, &ipv6calc_anon_set_list[i], sizeof(s_ipv6calc_anon_set));
+			return 0;
+		};
+	};
+
+	return 1;
+};
+
+/*
+ * create string of anonymization settings
+ *
+ * in : *string = string to be filled
+ *      s_ipv6calc_anon_set = anonymization set
+ * return: void
+ */
+void libipv6calc_anon_infostring(char *string, const int stringlength, const s_ipv6calc_anon_set *ipv6calc_anon_set) {
+	char method_name[32] = "unknown";
+	int i;
+
+	for (i = 0; i < sizeof(ipv6calc_anon_methods) / sizeof(s_ipv6calc_anon_methods); i++) {
+		if (ipv6calc_anon_methods[i].method == ipv6calc_anon_set->method) {
+			snprintf(method_name, sizeof(method_name) - 1, "%s", ipv6calc_anon_methods[i].name);
+			break;
+		};
+	};
+
+	snprintf(string, stringlength - 1, "set=%s,mask-ipv6=%d,mask-ipv4=%d,mask-iid=%d,mask-mac=%d,method=%s", ipv6calc_anon_set->name, ipv6calc_anon_set->mask_ipv6, ipv6calc_anon_set->mask_ipv4, ipv6calc_anon_set->mask_iid, ipv6calc_anon_set->mask_mac, method_name);
+
+	return;
+};
