@@ -1,8 +1,8 @@
 /*
  * Project    : ipv6calc
  * File       : libmac.c
- * Version    : $Id: libmac.c,v 1.14 2011/11/26 16:07:23 peter Exp $
- * Copyright  : 2001-2011 by Peter Bieringer <pb (at) bieringer.de>
+ * Version    : $Id: libmac.c,v 1.22 2013/04/13 17:34:28 ds6peter Exp $
+ * Copyright  : 2001-2013 by Peter Bieringer <pb (at) bieringer.de>
  *
  * Information:
  *  Function library MAC address handling
@@ -14,8 +14,10 @@
 #include "libipv6calc.h"
 #include "libmac.h"
 #include "ipv6calctypes.h"
+#include "libipv6calc.h"
+#include "libipv6calcdebug.h"
 
-static char ChSet[] = "0123456789abcdefABCDEF:- ";
+static char ChSet[] = "0123456789abcdefABCDEF:- .";
 
 /* function 48-bit MAC address to MACaddr_structure
  *
@@ -24,8 +26,8 @@ static char ChSet[] = "0123456789abcdefABCDEF:- ";
  * ret: ==0: ok, !=0: error
  */
 #define DEBUG_function_name "libmac/mac_to_macaddrstruct"
-int mac_to_macaddrstruct(char *addrstring, char *resultstring, ipv6calc_macaddr *macaddrp) {
-	int retval = 1, result, i, ccolons = 0, cdashes = 0, cspaces = 0;
+int mac_to_macaddrstruct(const char *addrstring, char *resultstring, ipv6calc_macaddr *macaddrp) {
+	int retval = 1, result, i, ccolons = 0, cdashes = 0, cspaces = 0, cdots = 0;
 	size_t cnt;
 	int temp[6];
 
@@ -45,7 +47,7 @@ int mac_to_macaddrstruct(char *addrstring, char *resultstring, ipv6calc_macaddr
 		
 	};
 
-	/* count ":" or "-" or " " must be 5 or 1 x "-" */
+	/* count ":" or "-" or " " or " " must be 5 or 1 x "-" */
 	for (i = 0; i < (int) strlen(addrstring); i++) {
 		if (addrstring[i] == ':') {
 			ccolons++;
@@ -53,10 +55,18 @@ int mac_to_macaddrstruct(char *addrstring, char *resultstring, ipv6calc_macaddr
 			cdashes++;
 		} else if (addrstring[i] == ' ') {
 			cspaces++;
+		} else if (addrstring[i] == '.') {
+			cdots++;
 		};
 	};
 
-	if ( ! ( (ccolons == 5 && cdashes == 0 && cspaces == 0) || (ccolons == 0 && cdashes == 5 && cspaces == 0)  || (ccolons == 0 && cdashes == 0 && cspaces == 5) || (ccolons == 0 && cdashes == 1 && cspaces == 0 && strlen(addrstring) == 13) || (ccolons == 0 && cdashes == 0 && cspaces == 0 && strlen(addrstring) == 12)) ) {
+	if ( ! ( (ccolons == 5 && cdashes == 0 && cspaces == 0 && cdots == 0)
+		   || (ccolons == 0 && cdashes == 5 && cspaces == 0 && cdots == 0)
+		   || (ccolons == 0 && cdashes == 0 && cspaces == 5 && cdots == 0)
+		   || (ccolons == 0 && cdashes == 0 && cspaces == 0 && cdots == 2)
+		   || (ccolons == 0 && cdashes == 1 && cspaces == 0 && strlen(addrstring) == 13 && cdots == 0)
+		   || (ccolons == 0 && cdashes == 0 && cspaces == 0 && strlen(addrstring) == 12 && cdots == 0))
+	   ) {
 		snprintf(resultstring, NI_MAXHOST - 1, "Error, given MAC address '%s' is not valid (number of colons/dashes/spaces is not 5 or number of dashes is not 1)!", addrstring);
 		retval = 1;
 		return (retval);
@@ -71,6 +81,8 @@ int mac_to_macaddrstruct(char *addrstring, char *resultstring, ipv6calc_macaddr
 		result = sscanf(addrstring, "%2x%2x%2x-%2x%2x%2x", &temp[0], &temp[1], &temp[2], &temp[3], &temp[4], &temp[5]);
 	} else if ( cspaces == 5 ) {
 		result = sscanf(addrstring, "%x %x %x %x %x %x", &temp[0], &temp[1], &temp[2], &temp[3], &temp[4], &temp[5]);
+	} else if ( cdots == 2 ) {
+		result = sscanf(addrstring, "%2x%2x.%2x%2x.%2x%2x", &temp[0], &temp[1], &temp[2], &temp[3], &temp[4], &temp[5]);
 	} else if ( cdashes == 0 ) {
 		result = sscanf(addrstring, "%2x%2x%2x%2x%2x%2x", &temp[0], &temp[1], &temp[2], &temp[3], &temp[4], &temp[5]);
 	} else {
@@ -106,6 +118,25 @@ int mac_to_macaddrstruct(char *addrstring, char *resultstring, ipv6calc_macaddr
 };
 #undef DEBUG_function_name
 
+/* compatible name */
+int addr_to_macaddrstruct(const char *addrstring, char *resultstring, ipv6calc_macaddr *macaddrp) {
+	return (mac_to_macaddrstruct(addrstring, resultstring, macaddrp));
+};
+
+/* 
+ * clear MACaddr
+ *
+ * mod: *addrstring = MAC address
+ */
+void mac_clear(ipv6calc_macaddr *macaddrp) {
+	int i;
+
+	for ( i = 0; i <= 5; i++ ) {
+		macaddrp->addr[i] = 0;
+	};  
+
+	return;
+};
 
 /* 
  * clear MACaddr_structure
@@ -113,11 +144,7 @@ int mac_to_macaddrstruct(char *addrstring, char *resultstring, ipv6calc_macaddr
  * mod: *addrstring = MAC address
  */
 void mac_clearall(ipv6calc_macaddr *macaddrp) {
-	int i;
-
-	for ( i = 0; i <= 5; i++ ) {
-		macaddrp->addr[i] = 0;
-	};  
+	mac_clear(macaddrp);
 
 	/* Clear valid flag */
 	macaddrp->flag_valid = 0;
@@ -150,3 +177,81 @@ int macaddrstruct_to_string(const ipv6calc_macaddr *macaddrp, char *resultstring
 };
 #undef DEBUG_function_name
 
+/* compatible name */
+int libmacaddr_macaddrstruct_to_string(const ipv6calc_macaddr *macaddrp, char *resultstring, const uint32_t formatoptions) {
+	return (macaddrstruct_to_string(macaddrp, resultstring, formatoptions));
+};
+
+
+/*
+ * clear filter MAC address
+ *
+ * in : *filter    = filter structure
+ */
+void macaddr_filter_clear(s_ipv6calc_filter_macaddr *filter) {
+	filter->active = 0;
+	filter->typeinfo_must_have = 0;
+	return;
+};
+
+
+
+/*
+ * filter MAC address
+ *
+ * in : *macaddrp = MAC address structure
+ * in : *filter    = filter structure
+ * ret: 0=match 1=not match
+ */
+int macaddr_filter(const ipv6calc_macaddr *macaddrp, const s_ipv6calc_filter_macaddr *filter) {
+
+	return 1;
+};
+
+
+/*
+ * anonymize MAC address
+ *
+ * in : *macaddrp = MAC address structure
+ *      mask = number of bits of mask
+ * ret: <void>
+ */
+void libmacaddr_anonymize(ipv6calc_macaddr *macaddrp, const unsigned int mask) {
+	int i, max, min, bit;
+
+	/* anonymize MAC address according to settings */
+
+	if (mask == 0) {
+		/* clear MAC address: 0:0:0:0:0:0 */
+		mac_clear(macaddrp);
+	} else if (mask == 48) {
+		/* nothing to do */
+	} else if (mask < 1 || mask > 48) {
+		/* should not happen here */
+		fprintf(stderr, "%s/%s: 'mask' has an unexpected illegal value: %d!\n", __FILE__, __func__, mask);
+		exit(EXIT_FAILURE);
+	} else {
+		/* complete byte mask */
+		max = 6;
+		min = mask / 8 + 2;
+
+                if ( (ipv6calc_debug ) != 0 ) {
+                        fprintf(stderr, "%s/%s: mask=%d -> complete byte mask from %d to %d\n", __FILE__, __func__, mask, max, min);
+                };
+
+		if (max > min) {
+			for (i = max; i >= min; i--) {
+				macaddrp->addr[i-1] = 0;
+			};
+		};
+
+		/* partial byte mask */
+		bit = (~ (0xffu >> (mask % 8))) & 0xff;
+                if ( (ipv6calc_debug ) != 0 ) {
+                        fprintf(stderr, "%s/%s: mask=%d -> partial byte mask %d with 0x%02x\n", __FILE__, __func__, mask, min-1, bit);
+                };
+
+		macaddrp->addr[min-2] &= bit;
+	};
+	return;
+};
